@@ -90,6 +90,35 @@ Thanks are due to D. Knuth for the long division algorithm.
 #include "save_vec.h"
 #include "processes.h"
 #include "memmgr.h"
+#include "rtsentry.h"
+#include "profiling.h"
+
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyAddArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolySubtractArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyMultiplyArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyDivideArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyRemainderArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyQuotRemArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2, PolyWord arg3);
+    POLYEXTERNALSYMBOL POLYSIGNED PolyCompareArbitrary(PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyGCDArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyLCMArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyGetLowOrderAsLargeWord(PolyObject *threadId, PolyWord arg);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyOrArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyAndArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyXorArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2);
+}
+
+static Handle add_longc(TaskData *taskData, Handle,Handle);
+static Handle sub_longc(TaskData *taskData, Handle,Handle);
+static Handle quot_rem_c(TaskData *taskData, Handle,Handle,Handle);
+static Handle or_longc(TaskData *taskData, Handle,Handle);
+static Handle and_longc(TaskData *taskData, Handle,Handle);
+static Handle xor_longc(TaskData *taskData, Handle,Handle);
+static Handle neg_longc(TaskData *taskData, Handle);
+
+static Handle gcd_arbitrary(TaskData *taskData, Handle,Handle);
+static Handle lcm_arbitrary(TaskData *taskData, Handle,Handle);
 
 // Number of bits in a Poly word.  N.B.  This is not necessarily the same as SIZEOF_VOIDP.
 #define BITS_PER_POLYWORD (SIZEOF_VOIDP*8)
@@ -97,7 +126,7 @@ Thanks are due to D. Knuth for the long division algorithm.
 #ifdef USE_GMP
 #if (BITS_PER_POLYWORD > GMP_LIMB_BITS)
 // We're assuming that every GMP limb occupies at least one word
-#error "Size of GMP limb is less than a Poly word"
+#error "Size of GMP limb is less than makemaka Poly word"
 #endif
 #endif
 
@@ -421,7 +450,7 @@ Handle Make_arbitrary_precision(TaskData *taskData, unsigned long uval)
     return ArbitraryPrecionFromUnsigned(taskData, uval);
 }
 
-#if SIZEOF_LONG_LONG != 0
+#ifdef HAVE_LONG_LONG
 #if SIZEOF_LONG_LONG <= SIZEOF_VOIDP
 Handle Make_arbitrary_precision(TaskData *taskData, long long val)
 {
@@ -1164,21 +1193,21 @@ void getFileTimeFromArb(TaskData *taskData, Handle numHandle, PFILETIME ft)
 #endif
 
 /* compare_unsigned is passed LONG integers only */
-static int compare_unsigned(Handle x, Handle y)
+static int compare_unsigned(PolyWord x, PolyWord y)
 {
 #ifdef USE_GMP
-    mp_size_t lx = numLimbs(DEREFWORD(x));
-    mp_size_t ly = numLimbs(DEREFWORD(y));
+    mp_size_t lx = numLimbs(x);
+    mp_size_t ly = numLimbs(y);
 
     if (lx != ly)  /* u > v if u longer than v */
     {
         return (lx > ly ? 1 : -1);
     }
-    return mpn_cmp(DEREFLIMBHANDLE(x), DEREFLIMBHANDLE(y), lx);
+    return mpn_cmp((mp_limb_t *)x.AsCodePtr(), (mp_limb_t *)y.AsCodePtr(), lx);
 #else
     /* First look at the lengths */
-    POLYUNSIGNED lx = get_length(DEREFWORD(x));
-    POLYUNSIGNED ly = get_length(DEREFWORD(y));
+    POLYUNSIGNED lx = get_length(x);
+    POLYUNSIGNED ly = get_length(y);
 
     if (lx != ly)  /* u > v if u longer than v */
     {
@@ -1186,8 +1215,8 @@ static int compare_unsigned(Handle x, Handle y)
     }
 
     // Same length - look at the values. */
-    byte *u = DEREFBYTEHANDLE(x);
-    byte *v = DEREFBYTEHANDLE(y);
+    byte *u = x.AsCodePtr();
+    byte *v = y.AsCodePtr();
 
     POLYUNSIGNED i = lx;
     while (i > 0)
@@ -1203,60 +1232,45 @@ static int compare_unsigned(Handle x, Handle y)
 #endif
 }
 
-int compareLong(TaskData *taskData, Handle y, Handle x)
+int compareLong(PolyWord y, PolyWord x)
 {
     // Test if the values are bitwise equal.  If either is short
     // this is the only case where the values could be equal.
-    if (DEREFWORD(x) == DEREFWORD(y)) // Equal
+    if (x == y) // Equal
         return 0;
 
-    if (IS_INT(DEREFWORD(x)))
+    if (x.IsTagged())
     {
         // x is short.
-        if (IS_INT(DEREFWORD(y))) {
+        if (y.IsTagged()) {
             // Both short.  We've already tested for equality.
-            if (UNTAGGED(DEREFWORD(x)) < UNTAGGED(DEREFWORD(y)))
+            if (x.UnTagged() < y.UnTagged())
                 return -1; // Less
             else return 1; // Greater
         }
         // y is not short.  Just test the sign.  If it's negative
         // it must be less than any short value and if it's positive
         // it must be greater.
-        if (OBJ_IS_NEGATIVE(GetLengthWord(DEREFWORD(y))))
+        if (OBJ_IS_NEGATIVE(GetLengthWord(y)))
             return 1; // x is greater
         else return -1; // x is less
     }
 
     // x is not short
-    if (IS_INT(DEREFWORD(y)))
+    if (y.IsTagged())
     {
         // y is short.  Just test the sign of x
-        if (OBJ_IS_NEGATIVE(GetLengthWord(DEREFWORD(x))))
+        if (OBJ_IS_NEGATIVE(GetLengthWord(x)))
             return -1; // x is less
         else return 1; // x is greater
     }
 
-    /* Convert to long form */
-#if USE_GMP
-    PolyWord    x_extend[1+WORDS(sizeof(mp_limb_t))];
-    PolyWord    y_extend[1+WORDS(sizeof(mp_limb_t))];
-#else
-    PolyWord    x_extend[2], y_extend[2];
-#endif
-    SaveVecEntry x_extend_addr = SaveVecEntry(PolyWord::FromStackAddr(&(x_extend[1])));
-    Handle x_ehandle = &x_extend_addr;
-    SaveVecEntry y_extend_addr = SaveVecEntry(PolyWord::FromStackAddr(&(y_extend[1])));
-    Handle y_ehandle = &y_extend_addr;
-
-    int sign_x, sign_y;
-    Handle long_x = get_long(x, x_ehandle, &sign_x); /* Convert to long form */
-    Handle long_y = get_long(y, y_ehandle, &sign_y);
-
-    if (sign_x >= 0) /* x is positive */
+    // Must both be long.  We may be able to determine the result based purely on the sign bits.
+    if (! OBJ_IS_NEGATIVE(GetLengthWord(x))) /* x is positive */
     {
-        if (sign_y >= 0) /* y also positive */
+        if (! OBJ_IS_NEGATIVE(GetLengthWord(y))) /* y also positive */
         {
-            return compare_unsigned(long_x, long_y);
+            return compare_unsigned(x, y);
         }
         else /* y negative so x > y */
         {
@@ -1265,9 +1279,9 @@ int compareLong(TaskData *taskData, Handle y, Handle x)
     }
     else
     { /* x is negative */
-        if (sign_y < 0) /* y also negative */
+        if (OBJ_IS_NEGATIVE(GetLengthWord(y))) /* y also negative */
         {
-            return compare_unsigned(long_y, long_x);
+            return compare_unsigned(y, x);
         }
         else /* y positive so x < y */
         {
@@ -1276,7 +1290,6 @@ int compareLong(TaskData *taskData, Handle y, Handle x)
     }
 } /* compareLong */
 
-
 /* logical_long.  General purpose function for binary logical operations. */
 static Handle logical_long(TaskData *taskData, Handle x, Handle y, int signX, int signY,
                            unsigned(*op)(unsigned, unsigned))
@@ -1487,96 +1500,35 @@ Handle xor_longc(TaskData *taskData, Handle y, Handle x)
 }
 
 // Convert a long precision value to floating point
-double get_C_real(TaskData *, PolyWord x)
+double get_arbitrary_precision_as_real(PolyWord x)
 {
     if (IS_INT(x)) {
         POLYSIGNED t = UNTAGGED(x);
         return (double)t;
     }
+    double acc = 0;
+#if USE_GMP
+    mp_limb_t *u = (mp_limb_t *)(x.AsObjPtr());
+    mp_size_t lx = numLimbs(x);
+    for ( ; lx > 0; lx--) {
+        int ll = sizeof(mp_limb_t);
+        for ( ; ll > 0 ; ll-- ) {
+            acc = acc * 256;
+        }
+        acc = acc + (double)u[lx-1];
+    }
+#else
     byte *u = (byte *)(x.AsObjPtr());
     POLYUNSIGNED lx = OBJECT_LENGTH(x)*sizeof(PolyWord);
-    double acc = 0;
     for( ; lx > 0; lx--) {
         acc = acc * 256 + (double)u[lx-1];
     }
+#endif
     if (OBJ_IS_NEGATIVE(GetLengthWord(x)))
         return -acc;
     else return acc;
 }
 
-/*
-These functions are used primarily during porting.  They handle both
-short and long forms of integers and are generally superseded by
-hand coded assembly code versions and the code generator.
-Some of them are retained in some code-generators to handle the
-long forms of integers.
-*/
-
-Handle equal_longc(TaskData *taskData, Handle y, Handle x)
-/* Returns 1 if the arguments are equal, otherwise 0. */
-{
-    bool c = compareLong(taskData, y, x) == 0;
-    return taskData->saveVec.push(c ? TAGGED(1) : TAGGED(0));
-}
-
-Handle not_equal_longc(TaskData *taskData, Handle y, Handle x)
-{
-    bool c = compareLong(taskData, y, x) != 0;
-    return taskData->saveVec.push(c ? TAGGED(1) : TAGGED(0));
-}
-
-Handle gt_longc(TaskData *taskData, Handle y, Handle x)
-{
-    bool c = (compareLong(taskData, y, x) == 1);
-
-    return taskData->saveVec.push(c ? TAGGED(1) : TAGGED(0));
-}
-
-Handle ls_longc(TaskData *taskData, Handle y, Handle x)
-{
-    bool c = (compareLong(taskData, y, x) == -1);
-
-    return taskData->saveVec.push(c ? TAGGED(1) : TAGGED(0));
-}
-
-Handle ge_longc(TaskData *taskData, Handle y, Handle x)
-{
-    bool c = compareLong(taskData, y, x) != -1;
-
-    return taskData->saveVec.push(c ? TAGGED(1) : TAGGED(0));
-}
-
-Handle le_longc(TaskData *taskData, Handle y, Handle x)
-{
-    bool c = compareLong(taskData, y, x) != 1;
-
-    return taskData->saveVec.push(c ? TAGGED(1) : TAGGED(0));
-}
-
-/* Return the low-order bits of an integer whether it is long or
-   short form. */
-Handle int_to_word_c(TaskData *taskData, Handle x)
-{
-    /* If it's already short it's easy. */
-    if (IS_INT(DEREFWORD(x)))
-        return x;
-
-#ifdef USE_GMP
-    // It may be big- or little-endian.
-    POLYUNSIGNED r = (POLYUNSIGNED)*DEREFLIMBHANDLE(x);
-#else
-    // Little-endian.
-    byte    *u = DEREFBYTEHANDLE(x);
-    POLYUNSIGNED r = 0;
-    for (unsigned i=0; i < sizeof(PolyWord); i++)
-    {
-        r |= (POLYUNSIGNED)u[i] << (8*i);
-    }
-#endif
-    if (OBJ_IS_NEGATIVE(x->Word().AsObjPtr()->LengthWord()))
-        r = 0-r; // Use 0-r rather than -r since it's an unsigned value.
-    return taskData->saveVec.push(TAGGED(r));
-}
 
 /*  Arbitrary precision GCD function.  This is really included to make
     use of GMP's GCD function that selects an algorithm based on the
@@ -1733,7 +1685,7 @@ Handle gcd_arbitrary(TaskData *taskData, Handle x, Handle y)
     x = absValue(taskData, x);
     y = absValue(taskData, y);
 
-    if (compareLong(taskData, y, x) < 0)
+    if (compareLong(y->Word(), x->Word()) < 0)
         return gxd(taskData, y, x);
     else return gxd(taskData, x, y);
 }
@@ -1747,4 +1699,317 @@ Handle lcm_arbitrary(TaskData *taskData, Handle x, Handle y)
     return mult_longc(taskData, x, div_longc(taskData, g, y));
 }
 
+POLYUNSIGNED PolyAddArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+    
+    if (profileMode == kProfileEmulation)
+        taskData->addProfileCount(1);
 
+    try {
+        // Could raise an exception if out of memory.
+        result = add_longc(taskData, pushedArg2, pushedArg1);
+    }
+    catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolySubtractArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+    
+    if (profileMode == kProfileEmulation)
+        taskData->addProfileCount(1);
+
+    try {
+        result = sub_longc(taskData, pushedArg2, pushedArg1);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyMultiplyArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+    
+    if (profileMode == kProfileEmulation)
+        taskData->addProfileCount(1);
+
+    try {
+        result = mult_longc(taskData, pushedArg2, pushedArg1);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyDivideArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+    
+    if (profileMode == kProfileEmulation)
+        taskData->addProfileCount(1);
+
+    try {
+        // May raise divide exception
+        result = div_longc(taskData, pushedArg2, pushedArg1);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyRemainderArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+    
+    if (profileMode == kProfileEmulation)
+        taskData->addProfileCount(1);
+
+    try {
+        result = rem_longc(taskData, pushedArg2, pushedArg1);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyQuotRemArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2, PolyWord arg3)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle pushedArg3 = taskData->saveVec.push(arg3);
+    
+    if (profileMode == kProfileEmulation)
+        taskData->addProfileCount(1);
+
+    try {
+        quot_rem_c(taskData, pushedArg3, pushedArg2, pushedArg1);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    return 0; // Result is unit
+}
+
+// This can be a fast call.  It does not need to allocate or use handles.
+POLYSIGNED PolyCompareArbitrary(PolyWord arg1, PolyWord arg2)
+{
+    return TAGGED(compareLong(arg2, arg1)).AsSigned();
+}
+
+POLYUNSIGNED PolyGCDArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+
+    try {
+        result = gcd_arbitrary(taskData, pushedArg2, pushedArg1);
+        // Generally shouldn't raise an exception but might run out of store.
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyLCMArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+
+    try {
+        result = lcm_arbitrary(taskData, pushedArg2, pushedArg1);
+        // Generally shouldn't raise an exception but might run out of store.
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+// Extract the low order part of an arbitrary precision value as a boxed LargeWord.word
+// value.  If the value is negative it is treated as a twos complement value.
+// This is used Word.fromLargeInt and LargeWord.fromLargeInt with long-form
+// arbitrary precision values.
+POLYUNSIGNED PolyGetLowOrderAsLargeWord(PolyObject *threadId, PolyWord arg)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    POLYSIGNED p = 0;
+
+    if (arg.IsTagged())
+        p = arg.UnTagged();
+    else
+    {
+        bool negative = OBJ_IS_NEGATIVE(GetLengthWord(arg)) ? true : false;
+#ifdef USE_GMP
+        mp_limb_t c = *(mp_limb_t*)arg.AsCodePtr();
+        p = c;
+#else
+        POLYUNSIGNED length = get_length(arg);
+        if (length > sizeof(PolyWord)) length = sizeof(PolyWord);
+        byte *ptr = arg.AsCodePtr();
+        while (length--)
+        {
+            p = (p << 8) | ptr[length];
+        }
+#endif
+        if (negative) p = -p;
+    }
+
+    Handle result = alloc_and_save(taskData, 1, F_BYTE_OBJ);
+    result->WordP()->Set(0, PolyWord::FromUnsigned(p));
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    return result->Word().AsUnsigned();
+
+
+}
+
+POLYUNSIGNED PolyOrArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+
+    try {
+        // Could raise an exception if out of memory.
+        result = or_longc(taskData, pushedArg2, pushedArg1);
+    }
+    catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyAndArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+
+    try {
+        // Could raise an exception if out of memory.
+        result = and_longc(taskData, pushedArg2, pushedArg1);
+    }
+    catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyXorArbitrary(PolyObject *threadId, PolyWord arg1, PolyWord arg2)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg1 = taskData->saveVec.push(arg1);
+    Handle pushedArg2 = taskData->saveVec.push(arg2);
+    Handle result = 0;
+
+    try {
+        // Could raise an exception if out of memory.
+        result = xor_longc(taskData, pushedArg2, pushedArg1);
+    }
+    catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+struct _entrypts arbitraryPrecisionEPT[] =
+{
+    { "PolyAddArbitrary",               (polyRTSFunction)&PolyAddArbitrary},
+    { "PolySubtractArbitrary",          (polyRTSFunction)&PolySubtractArbitrary},
+    { "PolyMultiplyArbitrary",          (polyRTSFunction)&PolyMultiplyArbitrary},
+    { "PolyDivideArbitrary",            (polyRTSFunction)&PolyDivideArbitrary},
+    { "PolyRemainderArbitrary",         (polyRTSFunction)&PolyRemainderArbitrary},
+    { "PolyQuotRemArbitrary",           (polyRTSFunction)&PolyQuotRemArbitrary},
+    { "PolyCompareArbitrary",           (polyRTSFunction)&PolyCompareArbitrary},
+    { "PolyGCDArbitrary",               (polyRTSFunction)&PolyGCDArbitrary},
+    { "PolyLCMArbitrary",               (polyRTSFunction)&PolyLCMArbitrary},
+    { "PolyGetLowOrderAsLargeWord",     (polyRTSFunction)&PolyGetLowOrderAsLargeWord},
+    { "PolyOrArbitrary",                (polyRTSFunction)&PolyOrArbitrary},
+    { "PolyAndArbitrary",               (polyRTSFunction)&PolyAndArbitrary},
+    { "PolyXorArbitrary",               (polyRTSFunction)&PolyXorArbitrary},
+
+    { NULL, NULL} // End of list.
+};
