@@ -4,7 +4,7 @@
     Copyright (c) 2000
         Cambridge University Technical Services Limited
 
-    Further modification Copyright 2015 David C. J. Matthews
+    Further modification Copyright 2015-16 David C. J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -35,7 +35,7 @@ extern Handle Make_arbitrary_precision(TaskData *taskData, unsigned long);
 extern Handle Make_arbitrary_precision(TaskData *taskData, int);
 extern Handle Make_arbitrary_precision(TaskData *taskData, unsigned);
 
-#if SIZEOF_LONG_LONG != 0
+#ifdef HAVE_LONG_LONG
 extern Handle Make_arbitrary_precision(TaskData *taskData, long long);
 extern Handle Make_arbitrary_precision(TaskData *taskData, unsigned long long);
 #endif
@@ -47,29 +47,11 @@ extern Handle Make_arb_from_pair_scaled(TaskData *taskData, unsigned hi, unsigne
 extern Handle Make_arb_from_Filetime(TaskData *taskData, const FILETIME &ft);
 #endif
 
-extern Handle add_longc(TaskData *taskData, Handle,Handle);
-extern Handle sub_longc(TaskData *taskData, Handle,Handle);
+// This is currently used in the interpreter.
 extern Handle mult_longc(TaskData *taskData, Handle,Handle);
+// Currently used in basicio.cpp
 extern Handle div_longc(TaskData *taskData, Handle,Handle);
 extern Handle rem_longc(TaskData *taskData, Handle,Handle);
-extern Handle quot_rem_c(TaskData *taskData, Handle,Handle,Handle);
-extern Handle or_longc(TaskData *taskData, Handle,Handle);
-extern Handle and_longc(TaskData *taskData, Handle,Handle);
-extern Handle xor_longc(TaskData *taskData, Handle,Handle);
-extern Handle neg_longc(TaskData *taskData, Handle);
-
-extern Handle equal_longc(TaskData *taskData, Handle y, Handle x);
-extern Handle not_equal_longc(TaskData *taskData, Handle y, Handle x);
-extern Handle gt_longc(TaskData *taskData, Handle y, Handle x);
-extern Handle ls_longc(TaskData *taskData, Handle y, Handle x);
-extern Handle ge_longc(TaskData *taskData, Handle y, Handle x);
-extern Handle le_longc(TaskData *taskData, Handle y, Handle x);
-extern Handle int_to_word_c(TaskData *taskData, Handle x);
-
-extern Handle gcd_arbitrary(TaskData *taskData, Handle,Handle);
-extern Handle lcm_arbitrary(TaskData *taskData, Handle,Handle);
-
-extern int              compareLong(TaskData *taskData, Handle,Handle);
 
 // Return a uintptr_t/intptr_t value.
 extern POLYUNSIGNED     getPolyUnsigned(TaskData *taskData, PolyWord);
@@ -83,10 +65,12 @@ extern unsigned         get_C_unsigned(TaskData *taskData, PolyWord);
 extern short            get_C_short(TaskData *taskData, PolyWord);
 extern int              get_C_int(TaskData *taskData, PolyWord);
 
-extern double           get_C_real(TaskData *taskData, PolyWord x);
+extern double           get_arbitrary_precision_as_real(PolyWord x);
 
 #if defined(_WIN32)
 void getFileTimeFromArb(TaskData *taskData, Handle numHandle, PFILETIME ft);
 #endif
 
+extern struct _entrypts arbitraryPrecisionEPT[];
+
 #endif
