@@ -60,8 +60,7 @@ bool Bitmap::Create(POLYUNSIGNED bits)
 {
     free(m_bits); // Any previous data
     size_t bytes = (bits+7) >> 3;
-    m_bits = (unsigned char*)malloc(bytes);
-    if (m_bits != 0) memset(m_bits, 0, bytes);
+    m_bits = (unsigned char*)calloc(bytes, sizeof(unsigned char));
     return m_bits != 0;
 }
 
@@ -108,22 +107,16 @@ void Bitmap::SetBits(POLYUNSIGNED bitno, POLYUNSIGNED length)
         length = stop_bit_index - 8;
     }
     
-    /* Invariant: 0 <= length */
-    ASSERT(length >= 0);
-    
     /* Set as many full bytes as possible */
-    while (8 <= length)
+    if (8 <= length)
     {
-        /* Invariant: 8 <= length */
-        byte_index ++;
-//        ASSERT(m_bits[byte_index] == 0);
-        m_bits[byte_index] = 0xff;
-        length -= 8;
-        /* Invariant: 0 <= length */
+        memset(m_bits + byte_index + 1, 0xff, length >> 3);
+        byte_index += length >> 3;
+        length &= 7;
     }
     
     /* Invariant: 0 <= length < 8 */
-    ASSERT(length >= 0 && length < 8);
+    ASSERT(length < 8);
     if (length == 0) return;
     
     /* Invariant: 0 < length < 8 */
@@ -241,3 +234,20 @@ POLYUNSIGNED Bitmap::CountSetBits(POLYUNSIGNED size) const
     return count;
 }
 
+// Find the last set bit before here.  Used to find the start of a code cell.
+// Returns zero if no bit is set.
+POLYUNSIGNED Bitmap::FindLastSet(POLYUNSIGNED bitno) const
+{
+    size_t byteno = bitno >> 3;
+    // Code cells are quite long so most of the bitmap will be zero.
+    if (m_bits[byteno] == 0)
+    {
+       do {
+            if (byteno == 0) return 0;
+            byteno--;
+        } while (m_bits[byteno] == 0);
+        bitno = (byteno << 3) + 7; // Set it to the last bit
+    }
+    while (bitno > 0 && ! TestBit(bitno)) bitno--;
+    return bitno;
+}
\ No newline at end of file
