@@ -1,7 +1,7 @@
 /*
     Title:  Header for diagnostics.c
 
-    Copyright (c) 2011-12, 2015 David C.J. Matthews
+    Copyright (c) 2011-12, 2015, 2017 David C.J. Matthews
 
     Copyright (c) 2000
         Cambridge University Technical Services Limited
@@ -44,16 +44,18 @@ extern void LogSize(POLYUNSIGNED wordSize);
 extern unsigned    debugOptions; // debugging  flags
 
 // Values for debugging flags
-#define DEBUG_CHECK_OBJECTS 0x001       // Check that addresses are valid.
-#define DEBUG_GC            0x002       // Summary GC debugging output
-#define DEBUG_MEMMGR        0x004       // Memory manager output
-#define DEBUG_GC_DETAIL     0x008       // Detailed GC debugging output
-#define DEBUG_THREADS       0x010       // Thread-related output
-#define DEBUG_GCTASKS       0x020       // GC task farm
-#define DEBUG_HEAPSIZE      0x040       // Heap resizing
-#define DEBUG_X             0x080       // X-Windows debugging
-#define DEBUG_SHARING       0x100       // Diagnostics for share-common-data
-#define DEBUG_CONTENTION    0x200       // Information about contended locks
-#define DEBUG_RTSCALLS      0x400       // Information about run-time calls.
+#define DEBUG_CHECK_OBJECTS 0x0001      // Check that addresses are valid.
+#define DEBUG_GC            0x0002      // Summary GC debugging output
+#define DEBUG_MEMMGR        0x0004      // Memory manager output
+#define DEBUG_GC_DETAIL     0x0008      // Detailed GC debugging output
+#define DEBUG_THREADS       0x0010      // Thread-related output
+#define DEBUG_GCTASKS       0x0020      // GC task farm
+#define DEBUG_HEAPSIZE      0x0040      // Heap resizing
+#define DEBUG_X             0x0080      // X-Windows debugging
+#define DEBUG_SHARING       0x0100      // Diagnostics for share-common-data
+#define DEBUG_CONTENTION    0x0200      // Information about contended locks
+#define DEBUG_RTSCALLS      0x0400      // Information about run-time calls. Not currently used.
+#define DEBUG_GC_ENHANCED   0x0800      // Intermediate level GC output
+#define DEBUG_SAVING        0x1000      // Saving state and exporting
 
 #endif
