@@ -108,7 +108,11 @@ typedef char TCHAR;
 #include "save_vec.h"
 #include "rts_module.h"
 #include "locking.h"
+#include "rtsentry.h"
 
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyForeignGeneral(PolyObject *threadId, PolyWord code, PolyWord arg);
+}
 
 /**********************************************************************
  *
@@ -258,9 +262,11 @@ static PLock volLock; // Mutex to protect vols.
 // ML memory but another thread could block waiting for the mutex.
 // N.B. raising an exception involves an allocation.
 
-#define FIRST_VOL 0
+// Index zero is special - this is always null.
+#define VOL_NULL_INDEX 0
+#define FIRST_VOL 1
 
-static POLYUNSIGNED num_vols = 0;
+static POLYUNSIGNED num_vols = 1;
 static POLYUNSIGNED next_vol = FIRST_VOL;
 
 
@@ -368,6 +374,9 @@ static void* DEREFVOL (TaskData *taskData, PolyWord v)
         info (("Invalid volatile -- bad magic number, index=<%" POLYUFMT ">\n", index));
         RAISE_EXN("Bad volatile magic number");
     }
+
+    // The Null vol is persistent and doesn't have an ML pointer
+    if (index == VOL_NULL_INDEX) return vols[index].C_pointer;
     
     if (index < num_vols) {
         if (vols[index].ML_pointer == v.AsObjPtr()) {
@@ -601,6 +610,13 @@ effect on Pass 3.
 
 **********************************************************************/
 
+struct _entrypts foreignEPT[] =
+{
+    { "PolyForeignGeneral",             (polyRTSFunction)&PolyForeignGeneral},
+
+    { NULL, NULL} // End of list.
+};
+
 class Foreign: public RtsModule
 {
 public:
@@ -623,14 +639,12 @@ void Foreign::Init()
 
 void Foreign::Start()
 {
+    // Set up the NULL entry
     static void *nullValue = 0;
-    PolyVolData *nullV = (PolyVolData*)IoEntry(POLY_SYS_foreign_null);
-    V_INDEX(nullV) = next_vol++;
-    MakeVolMagic(nullV);
-    ML_POINTER(nullV) = nullV;
-    C_POINTER(nullV) = &nullValue;
-    OWN_C_SPACE(nullV) = false; // Not freed
-    FINALISER(nullV) = 0; // No finaliser
+    vols[VOL_NULL_INDEX].ML_pointer = 0;
+    vols[VOL_NULL_INDEX].C_pointer = &nullValue;
+    vols[VOL_NULL_INDEX].Own_C_space = false; // Not freed
+    vols[VOL_NULL_INDEX].C_finaliser = 0; // No finaliser
 }
 
 void Foreign::Stop()
@@ -955,17 +969,12 @@ static Handle call_sym (TaskData *taskData, Handle symH, Handle argsH, Handle re
  *   
  **********************************************************************/
 
-/* This is backwards compatible with the old code in which toCchar and fromCchar
-   worked on "string" rather than "char".  Since single character strings and
-   chars have the same representation there's no need to change this.
-   DCJM 27/6/01. */
 static Handle toCchar (TaskData *taskData, Handle h)
 {
-    char s[2];
-    Poly_string_to_C(DEREFWORD(h),s,2);
-    mes(("<%c>\n", s[0]));
+    char ch = (char)get_C_int(taskData, UNHANDLE(h));
+    mes(("<%c>\n", ch));
     Handle res = vol_alloc_with_c_space(taskData, sizeof(char));
-    *(char*)DEREFVOL(taskData, UNHANDLE(res)) = s[0];
+    *(char*)DEREFVOL(taskData, UNHANDLE(res)) = ch;
     return res;
 }
 
@@ -973,7 +982,7 @@ static Handle fromCchar (TaskData *taskData, Handle h)
 {
     char c = *(char*)DEREFVOL(taskData, UNHANDLE(h));
     mes(("<%c>\n", c));
-    return SAVE(C_string_to_Poly(taskData, &c,1));
+    return Make_arbitrary_precision(taskData, c & 0xff);
 }
 
 
@@ -1199,7 +1208,7 @@ static Handle fromCbytes (TaskData *taskData, Handle h)
 { TRACE; {
     char* str = *(char**)DEREFVOL(taskData, DEREFHANDLE(h)->Get(0));
     size_t size = get_C_long(taskData, DEREFHANDLE(h)->Get(1));
-    if (str == NULL) return SAVE(EmptyString());
+    if (str == NULL) return SAVE(EmptyString(taskData));
     else return SAVE(C_string_to_Poly(taskData, str, size));
 }}
 
@@ -1497,7 +1506,7 @@ static Handle call_sym_and_convert (TaskData *taskData, Handle triple)
 static void callbackEntryPt(ffi_cif *cif, void *ret, void* args[], void *data)
 {
     uintptr_t cbIndex = (uintptr_t)data;
-    ASSERT(cbIndex >= 0 && cbIndex < callBackEntries);
+    ASSERT(cbIndex < callBackEntries);
     struct _cbStructEntry *cbEntry = &callbackTable[cbIndex];
     // We should get the task data for the thread that is running this code.
     // If this thread has been created by the foreign code we will have to
@@ -1641,9 +1650,14 @@ static Handle set_final (TaskData *taskData, Handle pair)
 }
 
 // Return the NULL vol.  This is a persistent vol which always contains null.
+// There is no associated back pointer.
 static Handle getNull(TaskData *taskData, Handle)
 {
-    return SAVE((PolyObject*)IoEntry(POLY_SYS_foreign_null));
+    PolyVolData* v = (PolyVolData*)alloc(taskData, VOL_BOX_SIZE, F_MUTABLE_BIT|F_BYTE_OBJ);
+    Handle result = SAVE(v);
+    V_INDEX(v) = VOL_NULL_INDEX;
+    MakeVolMagic(v);
+    return result;
 }
 
 /**********************************************************************
@@ -1710,7 +1724,7 @@ static type_hh_fun handlers[] =
 #define NUM_HANDLERS ((int)(sizeof(handlers)/sizeof(type_hh_fun)))
 
 
-Handle foreign_dispatch_c (TaskData *taskData, Handle args, Handle fcode_h)
+static Handle foreign_dispatch_c (TaskData *taskData, Handle args, Handle fcode_h)
 {
     int fcode = get_C_int(taskData, DEREFWORD(fcode_h));
     
@@ -1725,6 +1739,25 @@ Handle foreign_dispatch_c (TaskData *taskData, Handle args, Handle fcode_h)
     return (handlers[fcode])(taskData, args);
 }
 
+POLYEXTERNALSYMBOL POLYUNSIGNED PolyForeignGeneral(PolyObject *threadId, PolyWord code, PolyWord arg)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedArg = taskData->saveVec.push(arg);
+    Handle pushedCode = taskData->saveVec.push(code);
+    Handle result = 0;
+
+    try {
+        result = foreign_dispatch_c(taskData, pushedArg, pushedCode);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset);
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
 #else
 // The foreign function interface isn't available.
 #include "foreign.h"
@@ -1736,5 +1769,18 @@ Handle foreign_dispatch_c (TaskData *taskData, Handle args, Handle fcode_h)
     raise_exception_string(taskData, EXC_foreign, "The foreign function interface is not available on this platform");
 }
 
+POLYEXTERNALSYMBOL POLYUNSIGNED PolyForeignGeneral(PolyObject *threadId, PolyWord /*code*/, PolyWord /*arg*/)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    taskData->PreRTSCall();
+
+    try {
+        raise_exception_string(taskData, EXC_foreign, "The foreign function interface is not available on this platform");
+    } catch (...) { }  // Handle the C++ exception
+
+    taskData->PostRTSCall();
+    return TAGGED(0).AsUnsigned(); // Return unit since we're raising an exception
+}
+
 #endif
 
