@@ -4,6 +4,8 @@
     Copyright (c) 2000-7
         Cambridge University Technical Services Limited
 
+    Further development Copyright David C.J. Matthews 2010
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
@@ -23,21 +25,36 @@
 #ifndef GC_H_INCLUDED
 #define GC_H_INCLUDED
 
-class StackObject;
 #include "globals.h" // For POLYUNSIGNED
 
 class TaskData;
 
-extern void CopyStackFrame(StackObject *old_stack, StackObject *new_stack);
-
 // Make a request for a full garbage collection.
 extern void FullGC(TaskData *taskData);
 // Make a request for a partial garbage collection.
 extern bool QuickGC(TaskData *taskData, POLYUNSIGNED words_needed);
-extern void CreateHeap(void);
-extern POLYUNSIGNED GetPhysicalMemorySize(void);
+extern void CreateHeap();
+
+extern void FullGCForShareCommonData(void);
 
-extern unsigned gc_phase;
 extern bool convertedWeak;
 
+// Multi-thread GC.
+extern void initialiseMarkerTables();
+
+// The task farm for the GC.  The threads are left waiting for the GC,
+class GCTaskFarm;
+extern GCTaskFarm *gpTaskFarm;
+
+extern void CopyObjectToNewAddress(PolyObject *srcAddress, PolyObject *destAddress, POLYUNSIGNED L);
+
+extern bool RunQuickGC(const POLYUNSIGNED wordsRequiredToAllocate);
+
+// GC Phases.
+extern void GCSharingPhase(void);
+extern void GCMarkPhase(void);
+extern void GCheckWeakRefs(void);
+extern void GCCopyPhase(void);
+extern void GCUpdatePhase(void);
+
 #endif
