@@ -69,7 +69,7 @@ void MTGCCheckWeakRef::ScanRuntimeAddress(PolyObject **pt, RtsStrength weak)
     if (weak == STRENGTH_STRONG)
         return;
 
-    LocalMemSpace *space = gMem.LocalSpaceForAddress(w.AsStackAddr());
+    LocalMemSpace *space = gMem.LocalSpaceForAddress(w.AsStackAddr()-1);
     if (space == 0)
         return; // Not in local area
     // If it hasn't been marked set it to zero.
@@ -90,7 +90,7 @@ void MTGCCheckWeakRef::ScanAddressesInObject(PolyObject *obj, POLYUNSIGNED L)
         PolyWord someAddr = baseAddr[i];
         if (someAddr.IsDataPtr())
         {
-            LocalMemSpace *someSpace = gMem.LocalSpaceForAddress(someAddr.AsAddress());
+            LocalMemSpace *someSpace = gMem.LocalSpaceForAddress(someAddr.AsStackAddr()-1);
             if (someSpace != 0)
             {
                 PolyObject *someObj = someAddr.AsObjPtr();
@@ -108,7 +108,7 @@ void MTGCCheckWeakRef::ScanAddressesInObject(PolyObject *obj, POLYUNSIGNED L)
                 else
                 {
                     // Usual case: the contents of the SOME cell is the address of a ref.
-                    LocalMemSpace *space = gMem.LocalSpaceForAddress(refAddress.AsAddress());
+                    LocalMemSpace *space = gMem.LocalSpaceForAddress(refAddress.AsStackAddr());
                     if (space != 0) // If the ref is permanent it's always there.
                     {
                         POLYUNSIGNED new_bitno = space->wordNo(refAddress.AsStackAddr());
@@ -135,16 +135,16 @@ void MTGCCheckWeakRef::ScanAddressesInObject(PolyObject *obj, POLYUNSIGNED L)
 // because it also scans the area we're collecting.
 void MTGCCheckWeakRef::ScanAreas(void)
 {
-    for (unsigned i = 0; i < gMem.nlSpaces; i++)
+    for (std::vector<LocalMemSpace*>::iterator i = gMem.lSpaces.begin(); i < gMem.lSpaces.end(); i++)
     {
-        LocalMemSpace *space = gMem.lSpaces[i];
+        LocalMemSpace *space = *i;
         if (space->isMutable)
             ScanAddressesInRegion(space->lowestWeak, space->highestWeak);
     }
     // Scan the permanent mutable areas.
-    for (unsigned j = 0; j < gMem.npSpaces; j++)
+    for (std::vector<PermanentMemSpace*>::iterator i = gMem.pSpaces.begin(); i < gMem.pSpaces.end(); i++)
     {
-        MemSpace *space = gMem.pSpaces[j];
+        MemSpace *space = *i;
         if (space->isMutable)
             ScanAddressesInRegion(space->lowestWeak, space->highestWeak);
     }
