@@ -1,12 +1,11 @@
 /*
     Title:      Mutex and Condition Variable library.
 
-    Copyright (c) 2007, 2012 David C. J. Matthews
+    Copyright (c) 2007, 2012, 2015 David C. J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,9 +30,6 @@
 #define HAVE_PTHREAD 1
 #include <pthread.h>
 #elif (defined(HAVE_WINDOWS_H))
-// We need the next define to get TryEnterCriticalSection and
-// InitializeCriticalSectionAndSpinCount.
-#define _WIN32_WINNT 0x0500
 #include <windows.h>
 #endif
 
@@ -85,7 +81,7 @@ PLock::PLock(const char *n): lockName(n), lockCount(0)
 #ifdef HAVE_PTHREAD
     pthread_mutex_init(&lock, 0);
 #elif defined(HAVE_WINDOWS_H)
-    InitializeCriticalSectionAndSpinCount(&lock, 12000);
+    InitializeCriticalSection(&lock);
 #endif
 }
 
@@ -155,8 +151,7 @@ PCondVar::PCondVar()
 #ifdef HAVE_PTHREAD
     pthread_cond_init(&cond, NULL);
 #elif defined(HAVE_WINDOWS_H)
-    // Create a manually set event initially set.
-    cond = CreateEvent(NULL, TRUE, TRUE, NULL);
+    InitializeConditionVariable(&cond);
 #endif
 }
 
@@ -164,8 +159,6 @@ PCondVar::~PCondVar()
 {
 #ifdef HAVE_PTHREAD
     pthread_cond_destroy(&cond);
-#elif defined(HAVE_WINDOWS_H)
-    CloseHandle(cond);
 #endif
 }
 
@@ -175,19 +168,7 @@ void PCondVar::Wait(PLock *pLock)
 #ifdef HAVE_PTHREAD
     pthread_cond_wait(&cond, &pLock->lock);
 #elif defined(HAVE_WINDOWS_H)
-    // This code will NOT work as a general implementation of a
-    // condition variable.  It works provided we use it carefully.
-    // We only use this in the situation where all the threads
-    // that can wait on this condvar have waited before we then
-    // release all of them. 
-    ResetEvent(cond); // Do this with the lock held.
-    // Can now release the lock.  It doesn't matter that releasing
-    // the lock and waiting isn't atomic because the event is manually
-    // reset.  If another thread sets the event before we call WFSO we'll
-    // simply return immediately.
-    LeaveCriticalSection(&pLock->lock);
-    WaitForSingleObject(cond, INFINITE);
-    EnterCriticalSection(&pLock->lock);
+    SleepConditionVariableCS(&cond, &pLock->lock, INFINITE);
 #endif
 }
 
@@ -227,6 +208,8 @@ void PCondVar::WaitUntil(PLock *pLock, const timespec *time)
 #endif
 
 // Wait for a number of milliseconds.  Used within the RTS.  Drops the lock and reaquires it.
+// Returns true if the return was because the condition variable had been signalled.
+// Returns false if the timeout expired or there was an error.
 bool PCondVar::WaitFor(PLock *pLock, unsigned milliseconds)
 {
 #ifdef HAVE_PTHREAD
@@ -243,11 +226,8 @@ bool PCondVar::WaitFor(PLock *pLock, unsigned milliseconds)
     }
     return pthread_cond_timedwait(&cond, &pLock->lock, &waitTime) == 0;
 #elif defined(HAVE_WINDOWS_H)
-    ResetEvent(cond); // Do this with the lock held.
-    LeaveCriticalSection(&pLock->lock);
-    DWORD dwResult = WaitForSingleObject(cond, milliseconds);
-    EnterCriticalSection(&pLock->lock);
-    return dwResult == WAIT_OBJECT_0;
+    // SleepConditionVariableCS returns zero on error or timeout.
+    return SleepConditionVariableCS(&cond, &pLock->lock, milliseconds) != 0;
 #else
     return true; // Single-threaded.  Return immediately.
 #endif
@@ -259,9 +239,7 @@ void PCondVar::Signal(void)
 #ifdef HAVE_PTHREAD
     pthread_cond_broadcast(&cond);
 #elif defined(HAVE_WINDOWS_H)
-    // N.B.  This assumes that we have the same lock that is used
-    // in Wait and WaitFor otherwise we set the event before the reset.
-    SetEvent(cond);
+    WakeAllConditionVariable(&cond);
 #endif
 }
 
