@@ -1,7 +1,7 @@
 /*
     Title:  memmgr.cpp   Memory segment manager
 
-    Copyright (c) 2006-7, 2011-12, 2016 David C. J. Matthews
+    Copyright (c) 2006-7, 2011-12, 2016-17 David C. J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -322,6 +322,8 @@ PermanentMemSpace* MemMgr::NewExportSpace(POLYUNSIGNED size, bool mut, bool noOv
         if (space->bottom == 0)
         {
             delete space;
+            if (debugOptions & DEBUG_MEMMGR)
+                Log("MMGR: New export %smutable space: insufficient space\n", mut ? "" : "im");
             return 0;
         }
         space->isOwnSpace = true;
@@ -331,6 +333,11 @@ PermanentMemSpace* MemMgr::NewExportSpace(POLYUNSIGNED size, bool mut, bool noOv
         space->top = space->bottom + size;
         space->topPointer = space->bottom;
 
+        if (debugOptions & DEBUG_MEMMGR)
+            Log("MMGR: New export %smutable %s%sspace %p, size=%luk words, bottom=%p, top=%p\n", mut ? "" : "im",
+                noOv ? "no-overwrite " : "", code ? "code " : "", space,
+                space->spaceSize() / 1024, space->bottom, space->top);
+
         // Add to the table.
         try {
             AddTree(space);
@@ -339,11 +346,15 @@ PermanentMemSpace* MemMgr::NewExportSpace(POLYUNSIGNED size, bool mut, bool noOv
         catch (std::exception&) {
             RemoveTree(space);
             delete space;
+            if (debugOptions & DEBUG_MEMMGR)
+                Log("MMGR: New export %smutable space: Adding to tree failed\n", mut ? "" : "im");
             return 0;
         }
         return space;
     }
     catch (std::bad_alloc&) {
+        if (debugOptions & DEBUG_MEMMGR)
+            Log("MMGR: New export %smutable space: \"new\" failed\n", mut ? "" : "im");
         return 0;
     }
 }
@@ -401,6 +412,13 @@ bool MemMgr::PromoteExportSpaces(unsigned hierarchy)
                     for (PolyWord *ptr = space->bottom; ptr < space->top; )
                     {
                         PolyObject *obj = (PolyObject*)(ptr+1);
+                        // We may have forwarded this if this has been
+                        // copied to the exported area. Restore the original length word.
+                        if (obj->ContainsForwardingPtr())
+                        {
+                            PolyObject *forwardedTo = obj->FollowForwardingChain();
+                            obj->SetLengthWord(forwardedTo->LengthWord());
+                        }
                         if (obj->IsCodeObject())
                             space->headerMap.SetBit(ptr-space->bottom);
                         ptr += obj->Length() + 1;
@@ -688,6 +706,8 @@ void MemMgr::RemoveEmptyCodeAreas()
         PolyObject *start = (PolyObject *)(space->bottom+1);
         if (start->IsByteObject() && start->Length() == space->spaceSize()-1)
         {
+            if (debugOptions & DEBUG_MEMMGR)
+                Log("MMGR: Deleted code space %p\n", space);
             // We have an empty cell that fills the whole space.
             RemoveTree(space);
             delete(space);
@@ -1027,6 +1047,37 @@ void MemMgr::ReportHeapSizes(const char *phase)
     Log(" (%1.0f%%). Total space ", (float)inAlloc / (float)alloc * 100.0F);
     LogSize(spaceForHeap);
     Log(" %1.0f%% full.\n", (float)(inAlloc + inNonAlloc) / (float)spaceForHeap * 100.0F);
+    Log("Heap: Local spaces %u, permanent spaces %u, code spaces %u, stack spaces %u\n",
+        lSpaces.size(), pSpaces.size(), cSpaces.size(), sSpaces.size());
+    POLYUNSIGNED cTotal = 0, cOccupied = 0;
+    for (std::vector<CodeSpace*>::iterator c = cSpaces.begin(); c != cSpaces.end(); c++)
+    {
+        cTotal += (*c)->spaceSize();
+        PolyWord *pt = (*c)->bottom;
+        while (pt < (*c)->top)
+        {
+            pt++;
+            PolyObject *obj = (PolyObject*)pt;
+            if (obj->ContainsForwardingPtr())
+            {
+                obj = obj->FollowForwardingChain();
+                pt += obj->Length();
+            }
+            else
+            {
+                if (obj->IsCodeObject())
+                    cOccupied += obj->Length() + 1;
+                pt += obj->Length();
+            }
+        }
+    }
+    Log("Heap: Code area: total "); LogSize(cTotal); Log(" occupied: "); LogSize(cOccupied); Log("\n");
+    POLYUNSIGNED stackSpace = 0;
+    for (std::vector<StackSpace*>::iterator s = sSpaces.begin(); s != sSpaces.end(); s++)
+    {
+        stackSpace += (*s)->spaceSize();
+    }
+    Log("Heap: Stack area: total "); LogSize(stackSpace); Log("\n");
 }
 
 // Profiling - Find a code object or return zero if not found.
