@@ -1,12 +1,11 @@
 /*
     Title:  memmgr.h   Memory segment manager
 
-    Copyright (c) 2006-8, 2010-12 David C. J. Matthews
+    Copyright (c) 2006-8, 2010-12, 2016 David C. J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +23,7 @@
 
 #include "bitmap.h"
 #include "locking.h"
+#include <vector>
 
 // utility conversion macros
 #define Words_to_K(w) (w*sizeof(PolyWord))/1024
@@ -35,12 +35,12 @@ class GCTaskId;
 class TaskData;
 
 typedef enum {
-    ST_IO,          // The io area forms an interface with the RTS
     ST_PERMANENT,   // Permanent areas are part of the object code
                     // Also loaded saved state.
     ST_LOCAL,       // Local heaps contain volatile data
     ST_EXPORT,      // Temporary export area
-    ST_STACK        // ML Stack for a thread
+    ST_STACK,       // ML Stack for a thread
+    ST_CODE         // Code created in the current run
 } SpaceType;
 
 
@@ -75,6 +75,7 @@ public:
     SpaceType       spaceType;
     bool            isMutable;
     bool            isOwnSpace; // True if this has been allocated.
+    bool            isCode;
 
     PolyWord        *bottom;    // Bottom of area
     PolyWord        *top;       // Top of area.
@@ -109,14 +110,28 @@ public:
     PolyWord    *topPointer;
 
     Bitmap      shareBitmap; // Used in sharedata
+    Bitmap      profileCode; // Used when profiling
 
     friend class MemMgr;
 };
 
 #define NSTARTS 10
 
+// Markable spaces are used as the base class for local heap
+// spaces and code spaces.
+class MarkableSpace: public MemSpace
+{
+protected:
+    MarkableSpace();
+    virtual ~MarkableSpace() {}
+public:
+    PolyWord    *fullGCRescanStart; // Upper and lower limits for rescan during mark phase.
+    PolyWord    *fullGCRescanEnd;
+    PLock       spaceLock;        // Lock used to protect forwarding pointers
+};
+
 // Local areas can be garbage collected.
-class LocalMemSpace: public MemSpace
+class LocalMemSpace: public MarkableSpace
 {
 protected:
     LocalMemSpace();
@@ -133,13 +148,10 @@ public:
     PolyWord    *lowerAllocPtr;   // Allocation pointer. Objects are allocated BEFORE this.
 
     PolyWord    *fullGCLowerLimit;// Lowest object in area before copying.
-    PolyWord    *fullGCRescanStart; // Upper and lower limits for rescan during mark phase.
-    PolyWord    *fullGCRescanEnd;
     PolyWord    *partialGCTop;    // Value of upperAllocPtr before the current partial GC.
     PolyWord    *partialGCScan;   // Scan pointer used in minor GC
     PolyWord    *partialGCRootBase; // Start of the root objects.
     PolyWord    *partialGCRootTop;// Value of lowerAllocPtr after the roots have been copied.
-    PLock       spaceLock;        // Lock used to protect forwarding pointers
     GCTaskId    *spaceOwner;      // The thread that "owns" this space during a GC.
 
     Bitmap       bitmap;          /* bitmap with one bit for each word in the GC area. */
@@ -176,6 +188,17 @@ public:
     StackObject *stack()const { return (StackObject *)bottom; }
 };
 
+// Code Space.  These contain local code created by the compiler.
+class CodeSpace: public MarkableSpace
+{
+    public:
+        CodeSpace(PolyWord *start, POLYUNSIGNED spaceSize);
+
+    Bitmap  headerMap; // Map to find the headers during GC or profiling.
+    POLYUNSIGNED largestFree; // The largest free space in the area
+    PolyWord *firstFree; // The start of the first free space in the area.
+};
+
 class MemMgr
 {
 public:
@@ -189,10 +212,9 @@ public:
     // Create an entry for a permanent space.
     PermanentMemSpace *NewPermanentSpace(PolyWord *base, POLYUNSIGNED words,
         unsigned flags, unsigned index, unsigned hierarchy = 0);
-    // Create an entry for the IO space.
-    MemSpace   *InitIOSpace(PolyWord *base, POLYUNSIGNED words);
-    // Delete a local space
-    bool DeleteLocalSpace(LocalMemSpace *sp);
+    // Delete a local space.  Takes the iterator position in lSpaces and returns the
+    // iterator after deletion.
+    void DeleteLocalSpace(std::vector<LocalMemSpace*>::iterator &iter);
 
     // Allocate an area of the heap of at least minWords and at most maxWords.
     // This is used both when allocating single objects (when minWords and maxWords
@@ -203,6 +225,10 @@ public:
     PolyWord *AllocHeapSpace(POLYUNSIGNED words)
         { POLYUNSIGNED allocated = words; return AllocHeapSpace(words, allocated); }
 
+    CodeSpace *NewCodeSpace(POLYUNSIGNED size);
+    // Allocate space for code.  This is initially mutable to allow the code to be built.
+    PolyObject *AllocCodeSpace(PolyObject *initCell);
+
     // Check that a subsequent allocation will succeed.  Called from the GC to ensure
     bool CheckForAllocation(POLYUNSIGNED words);
 
@@ -222,18 +248,21 @@ public:
     bool DeleteStackSpace(StackSpace *space);
 
     // Create and delete export spaces
-    PermanentMemSpace *NewExportSpace(POLYUNSIGNED size, bool mut, bool noOv);
+    PermanentMemSpace *NewExportSpace(POLYUNSIGNED size, bool mut, bool noOv, bool code);
     void DeleteExportSpaces(void);
     bool PromoteExportSpaces(unsigned hierarchy); // Turn export spaces into permanent spaces.
     bool DemoteImportSpaces(void); // Turn previously imported spaces into local.
 
-    PermanentMemSpace *SpaceForIndex(unsigned index) const; // Return the space for a given index
+    PermanentMemSpace *SpaceForIndex(unsigned index); // Return the space for a given index
 
     // As a debugging check, write protect the immutable areas apart from during the GC.
     void ProtectImmutable(bool on);
 
     // Find a space that contains a given address.  This is called for every cell
     // during a GC so needs to be fast.,
+    // N.B.  This must be called on an address at the beginning or within the cell.
+    // Generally that means with a pointer to the length word.  Pointing at the
+    // first "data" word may give the wrong result if the length is zero.
     MemSpace *SpaceForAddress(const void *pt) const
     {
         uintptr_t t = (uintptr_t)pt;
@@ -252,6 +281,8 @@ public:
     }
 
     // Find a local address for a space.
+    // N.B.  The argument should generally be the length word.  See
+    // comment on SpaceForAddress.
     LocalMemSpace *LocalSpaceForAddress(const void *pt) const
     {
         MemSpace *s = SpaceForAddress(pt);
@@ -260,10 +291,6 @@ public:
         else return 0;
     }
 
-    bool IsIOPointer(const void *pt) const { return pt >= ioSpace->bottom && pt < ioSpace->top; }
-    bool IsLocalMutable(const void *pt) const
-    { LocalMemSpace *space = LocalSpaceForAddress(pt); return space != 0 && space->isMutable; }
-
     void SetReservation(POLYUNSIGNED words) { reservedSpace = words; }
 
     // In several places we assume that segments are filled with valid
@@ -275,36 +302,36 @@ public:
 
     // Remove unused local areas.
     void RemoveEmptyLocals();
+    // Remove unused code areas.
+    void RemoveEmptyCodeAreas();
 
     // Remove unused allocation areas to reduce the space below the limit.
     void RemoveExcessAllocation(POLYUNSIGNED words);
     void RemoveExcessAllocation() { RemoveExcessAllocation(spaceBeforeMinorGC); }
 
-    MemSpace *IoSpace() { return ioSpace; } // Return pointer to the IO space.
-
-    MemSpace *ioSpace; // The IO space
-
     // Table for permanent spaces
-    PermanentMemSpace **pSpaces;
-    unsigned npSpaces;
+    std::vector<PermanentMemSpace *> pSpaces;
 
     // Table for local spaces
-    LocalMemSpace **lSpaces;
-    unsigned nlSpaces;
+    std::vector<LocalMemSpace *> lSpaces;
 
     // Table for export spaces
-    PermanentMemSpace **eSpaces;
-    unsigned neSpaces;
+    std::vector<PermanentMemSpace *> eSpaces;
 
     // Table for stack spaces
-    StackSpace **sSpaces;
-    unsigned nsSpaces;
+    std::vector<StackSpace *> sSpaces;
     PLock stackSpaceLock;
 
+    // Table for code spaces
+    std::vector<CodeSpace *> cSpaces;
+    PLock codeSpaceLock;
 
     // Storage manager lock.
     PLock allocLock;
 
+    // Lock for creating new bitmaps for code profiling
+    PLock codeBitmapLock;
+
     unsigned nextIndex; // Used when allocating new permanent spaces.
 
     POLYUNSIGNED SpaceBeforeMinorGC() const { return spaceBeforeMinorGC; }
@@ -320,8 +347,14 @@ public:
 
     void ReportHeapSizes(const char *phase);
 
+    // Profiling - Find a code object or return zero if not found.
+    PolyObject *FindCodeObject(const byte *addr);
+    // Profiling - Free bitmaps to indicate start of an object.
+    void RemoveProfilingBitmaps();
+
 private:
     bool AddLocalSpace(LocalMemSpace *space);
+    bool AddCodeSpace(CodeSpace *space);
 
     POLYUNSIGNED reservedSpace;
     unsigned nextAllocator;
