@@ -4,10 +4,11 @@
     Copyright (c) 2000
         Cambridge University Technical Services Limited
 
+    Further development David C.J. Matthews 2016
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -62,6 +63,14 @@
 #include "bitmap.h"
 #include "memmgr.h"
 #include "mpoly.h"
+#include "processes.h"
+#include "rtsentry.h"
+
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyObjSize(PolyObject *threadId, PolyWord obj);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyShowSize(PolyObject *threadId, PolyWord obj);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyObjProfile(PolyObject *threadId, PolyWord obj);
+}
 
 extern FILE *polyStdout;
 
@@ -84,8 +93,6 @@ public:
 
     POLYUNSIGNED total_length;
     bool     show_size;
-    void     *io_bottom;
-    void     *io_top;
     VisitBitmap  **bitmaps;
     unsigned   nBitmaps;
     // Counts of objects of each size for mutable and immutable data.
@@ -100,26 +107,27 @@ ProcessVisitAddresses::ProcessVisitAddresses(bool show)
     // and gMem.lSpaces changing under our feet.
     PLocker lock(&gMem.allocLock);
 
-    MemSpace *ioSpace = gMem.IoSpace();
-    io_bottom    = ioSpace->bottom;
-    io_top       = ioSpace->top;
     total_length = 0;
     show_size    = show;
 
     // Create a bitmap for each of the areas apart from the IO area
-    nBitmaps = gMem.nlSpaces+gMem.npSpaces; //
+    nBitmaps = (unsigned)(gMem.lSpaces.size()+gMem.pSpaces.size()+gMem.cSpaces.size()); //
     bitmaps = new VisitBitmap*[nBitmaps];
     unsigned bm = 0;
-    unsigned j;
-    for (j = 0; j < gMem.npSpaces; j++)
+    for (std::vector<PermanentMemSpace*>::iterator i = gMem.pSpaces.begin(); i < gMem.pSpaces.end(); i++)
     {
-        MemSpace *space = gMem.pSpaces[j];
+        MemSpace *space = *i;
         // Permanent areas are filled with objects from the bottom.
         bitmaps[bm++] = new VisitBitmap(space->bottom, space->top);
     }
-    for (j = 0; j < gMem.nlSpaces; j++)
+    for (std::vector<LocalMemSpace*>::iterator i = gMem.lSpaces.begin(); i < gMem.lSpaces.end(); i++)
     {
-        LocalMemSpace *space = gMem.lSpaces[j];
+        LocalMemSpace *space = *i;
+        bitmaps[bm++] = new VisitBitmap(space->bottom, space->top);
+    }
+    for (std::vector<CodeSpace *>::iterator i = gMem.cSpaces.begin(); i < gMem.cSpaces.end(); i++)
+    {
+        CodeSpace *space = *i;
         bitmaps[bm++] = new VisitBitmap(space->bottom, space->top);
     }
     ASSERT(bm == nBitmaps);
@@ -262,9 +270,6 @@ POLYUNSIGNED ProcessVisitAddresses::ShowWord(PolyWord w)
     if (IS_INT(w))
         return 0; /* not a pointer */
     
-    if (w.AsAddress() >= io_bottom && w.AsAddress() < io_top)
-        return 0; /* IO segment */
-     
     if (w == PolyWord::FromUnsigned(0))
         return 0;
     
@@ -278,9 +283,8 @@ POLYUNSIGNED ProcessVisitAddresses::ShowWord(PolyWord w)
     
     PolyObject *p;
 
-    if (OBJ_IS_CODEPTR(w))
-        p = ObjCodePtrToPtr(w.AsCodePtr()); /* find beginning of the code object */
-    else p = w.AsObjPtr();
+    ASSERT(w.IsDataPtr());
+    p = w.AsObjPtr();
     
     /* Have we already visited this object? */
     if (bm->AlreadyVisited(p))
@@ -375,3 +379,60 @@ Handle ObjProfile(TaskData *taskData, Handle obj)
     fflush(polyStdout); /* We need this for Windows at least. */
     return Make_arbitrary_precision(taskData, process.total_length);
 }
+
+POLYUNSIGNED PolyObjSize(PolyObject *threadId, PolyWord obj)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+
+    ProcessVisitAddresses process(false);
+    process.ScanObjectAddress(obj.AsObjPtr());
+    Handle result = Make_arbitrary_precision(taskData, process.total_length);
+
+    taskData->PostRTSCall();
+    return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyShowSize(PolyObject *threadId, PolyWord obj)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+
+    ProcessVisitAddresses process(true);
+    process.ScanObjectAddress(obj.AsObjPtr());
+    fflush(polyStdout); /* We need this for Windows at least. */
+    Handle result = Make_arbitrary_precision(taskData, process.total_length);
+
+    taskData->PostRTSCall();
+    return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyObjProfile(PolyObject *threadId, PolyWord obj)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+
+    ProcessVisitAddresses process(false);
+    process.ScanObjectAddress(obj.AsObjPtr());
+    fprintf(polyStdout, "\nImmutable object sizes and counts\n");
+    printfprof(process.iprofile);
+    fprintf(polyStdout, "\nMutable object sizes and counts\n");
+    printfprof(process.mprofile);
+    fflush(polyStdout); /* We need this for Windows at least. */
+    Handle result = Make_arbitrary_precision(taskData, process.total_length);
+
+    taskData->PostRTSCall();
+    return result->Word().AsUnsigned();
+}
+
+struct _entrypts objSizeEPT[] =
+{
+    { "PolyObjSize",                    (polyRTSFunction)&PolyObjSize},
+    { "PolyShowSize",                   (polyRTSFunction)&PolyShowSize},
+    { "PolyObjProfile",                 (polyRTSFunction)&PolyObjProfile},
+
+    { NULL, NULL} // End of list.
+};
