@@ -2,13 +2,12 @@
     Title:     Export memory as a PE/COFF object
     Author:    David C. J. Matthews.
 
-    Copyright (c) 2006, 2013 David C. J. Matthews
+    Copyright (c) 2006, 2013, 2016 David C. J. Matthews
 
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,27 +31,31 @@
 class PECOFFExport: public Exporter, public ScanAddress
 {
 public:
-    PECOFFExport(): relocationCount(0), symbolCount(0) {}
+    PECOFFExport(): relocationCount(0), symbolNum(0) {}
 public:
     virtual void exportStore(void);
 
 private:
     // ScanAddress overrides
-    virtual void ScanConstant(byte *addrOfConst, ScanRelocationKind code);
+    virtual void ScanConstant(PolyObject *base, byte *addrOfConst, ScanRelocationKind code);
 
     // At the moment we should only get calls to ScanConstant.
     virtual PolyObject *ScanObjectAddress(PolyObject *base) { return base; }
     void alignFile(int align);
+    virtual void addExternalReference(void *addr, const char *name);
 
 private:
     void setRelocationAddress(void *p, DWORD *reloc);
     PolyWord createRelocation(PolyWord p, void *relocAddr);
-    void writeSymbol(const char *symbolName, __int32 value, int section, bool isExtern);
+    void writeSymbol(const char *symbolName, __int32 value, int section, bool isExtern, int symType=0);
 
     unsigned relocationCount;
-    unsigned symbolCount;
 
     ExportStringTable stringTable;
+
+    // Table and count for external references.
+    ExportStringTable externTable;
+    unsigned symbolNum;
 };
 
 #endif
