@@ -2,7 +2,7 @@
     Title:     Export and import memory in a portable format
     Author:    David C. J. Matthews.
 
-    Copyright (c) 2006-7, 2015-6 David C. J. Matthews
+    Copyright (c) 2006-7, 2015-7 David C. J. Matthews
 
 
     This library is free software; you can redistribute it and/or
@@ -402,6 +402,11 @@ PolyObject *SpaceAlloc::NewObj(POLYUNSIGNED objWords)
             size = objWords+1;
         size_t iSpace = size*sizeof(PolyWord);
         base = (PolyWord*)osMemoryManager->Allocate(iSpace, PERMISSION_READ|PERMISSION_WRITE|PERMISSION_EXEC);
+        if (base == 0)
+        {
+            fprintf(stderr, "Unable to allocate memory\n");
+            return 0;
+        }
         currentSize = iSpace/sizeof(PolyWord);
         used = 0;
     }
@@ -786,15 +791,6 @@ bool PImport::DoImport()
                         do ch = getc(f); while (ch == ' ');
                     }
                 }
-                // Adjust the byte count.  This is only necessary when importing the interpreted
-                // code into a machine with a different endian-ness from the exporter.
-                {
-                    POLYUNSIGNED l = 0;
-                    while (l < length && p->Get(l) != PolyWord::FromUnsigned(0)) l++;
-                    ASSERT(l < length);
-                    l++;
-                    p->Set(l, PolyWord::FromUnsigned(l*sizeof(PolyWord)));
-                }
                 break;
             }
 
