@@ -83,6 +83,18 @@
 #include "scanaddrs.h"
 #include "diagnostics.h"
 #include "reals.h"
+#include "rts_module.h"
+#include "rtsentry.h"
+
+static Handle poly_ffi (TaskData *taskData, Handle args, Handle code);
+
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyFFIGeneral(PolyObject *threadId, PolyWord code, PolyWord arg);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolySizeFloat();
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolySizeDouble();
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyFFIGetError(PolyWord addr);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyFFISetError(PolyWord err);
+}
 
 static struct _abiTable { const char *abiName; ffi_abi abiCode; } abiTable[] =
 {
@@ -162,19 +174,11 @@ static PLock callbackTableLock; // Mutex to protect table.
 static Handle mkAbitab(TaskData *taskData, void*, char *p);
 static void callbackEntryPt(ffi_cif *cif, void *ret, void* args[], void *data);
 
-static Handle toSysWord(TaskData *taskData, uintptr_t p)
-{
-    Handle result = alloc_and_save(taskData, 1, F_BYTE_OBJ);
-    *(uintptr_t*)(result->Word().AsCodePtr()) = p;
-    return result;
-}
-
 static Handle toSysWord(TaskData *taskData, void *p)
 {
-    return toSysWord(taskData, (uintptr_t)p);
+    return Make_sysword(taskData, (uintptr_t)p);
 }
 
-
 Handle poly_ffi(TaskData *taskData, Handle args, Handle code)
 {
     unsigned c = get_C_unsigned(taskData, code->Word());
@@ -242,7 +246,7 @@ Handle poly_ffi(TaskData *taskData, Handle args, Handle code)
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
             HMODULE hMod = *(HMODULE*)(args->WordP());
             if (! FreeLibrary(hMod))
-                raise_syscall(taskData, "FreeLibrary failed", -(int)GetLastError());
+                raise_syscall(taskData, "FreeLibrary failed", GetLastError());
 #else
             void *lib = *(void**)(args->WordP());
             if (dlclose(lib) != 0)
@@ -331,12 +335,11 @@ Handle poly_ffi(TaskData *taskData, Handle args, Handle code)
             // If we need the elements add space for the elements plus
             // one extra for the zero terminator.
             if (nElems != 0) space += (nElems+1) * sizeof(ffi_type *);
-            ffi_type *result = (ffi_type*)malloc(space);
+            ffi_type *result = (ffi_type*)calloc(1, space);
             // Raise an exception rather than returning zero.
             if (result == 0) raise_syscall(taskData, "Insufficient memory", ENOMEM);
             ffi_type **elem = 0;
             if (nElems != 0) elem = (ffi_type **)(result+1);
-            memset(result, 0, sizeof(ffi_type)); // Zero it in case they add fields
             result->size = size;
             result->alignment = align;
             result->type = type;
@@ -469,160 +472,6 @@ Handle poly_ffi(TaskData *taskData, Handle args, Handle code)
     }
 }
 
-// Load and store functions.  These are implemented in assembly code or the code-generator
-// so are only provided for the interpreter.
-// These functions all take a base address, an offset and an index.  The offset is
-// a byte addition to the base.  The index is added after multiplying by the size.
-Handle cmem_load_8(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    return Make_arbitrary_precision(taskData, baseAddr[index]);
-}
-
-Handle cmem_load_16(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    return Make_arbitrary_precision(taskData, ((uint16_t*)baseAddr)[index]);
-}
-
-#if (SIZEOF_VOIDP == 8)
-Handle cmem_load_32(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    // Return a tagged value in 64-bit mode.
-    return Make_arbitrary_precision(taskData, ((uint32_t*)baseAddr)[index]);
-}
-
-Handle cmem_load_64(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    // Box the result.
-    return toSysWord(taskData, ((uint64_t*)baseAddr)[index]);
-}
-#else
-Handle cmem_load_32(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    // Load 32-bit int - In 32-bit mode this needs to be boxed
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    return toSysWord(taskData, ((uint32_t*)baseAddr)[index]);
-}
-#endif
-
-Handle cmem_load_float(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    return real_result(taskData, ((float*)baseAddr)[index]);
-}
-
-Handle cmem_load_double(TaskData *taskData, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    return real_result(taskData, ((double*)baseAddr)[index]);
-}
-
-Handle cmem_store_8(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    uint8_t value = get_C_unsigned(taskData, valueH->Word());
-    baseAddr[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-
-Handle cmem_store_16(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    uint16_t value = get_C_unsigned(taskData, valueH->Word());
-    ((uint16_t*)baseAddr)[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-
-#if (SIZEOF_VOIDP == 8)
-Handle cmem_store_32(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    uint32_t value = get_C_unsigned(taskData, valueH->Word());
-    ((uint32_t*)baseAddr)[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-
-Handle cmem_store_64(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    // This is boxed.
-    uint64_t value = *(uint64_t*)(valueH->Word().AsAddress());
-    ((uint64_t*)baseAddr)[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-#else
-Handle cmem_store_32(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    // This is boxed in 32-bit mode.
-    uint32_t value = *(uint32_t*)(valueH->Word().AsAddress());
-    ((uint32_t*)baseAddr)[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-#endif
-
-Handle cmem_store_float(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    float value = (float)real_arg(taskData->saveVec.push(valueH->Word()));
-    ((float*)baseAddr)[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-
-Handle cmem_store_double(TaskData *taskData, Handle valueH, Handle indexH, Handle offsetH, Handle baseH)
-{
-    uint8_t *baseAddr =
-        *((uint8_t**)baseH->Word().AsAddress()) +
-        getPolySigned(taskData, offsetH->Word());
-    POLYSIGNED index = getPolySigned(taskData, indexH->Word());
-    double value = real_arg(taskData->saveVec.push(valueH->Word()));
-    ((double*)baseAddr)[index] = value;
-    return taskData->saveVec.push(TAGGED(0));
-}
-
-
 // Construct an entry in the ABI table.
 static Handle mkAbitab(TaskData *taskData, void *arg, char *p)
 {
@@ -641,7 +490,7 @@ static Handle mkAbitab(TaskData *taskData, void *arg, char *p)
 static void callbackEntryPt(ffi_cif *cif, void *ret, void* args[], void *data)
 {
     uintptr_t cbIndex = (uintptr_t)data;
-    ASSERT(cbIndex >= 0 && cbIndex < callBackEntries);
+    ASSERT(cbIndex < callBackEntries);
     // We should get the task data for the thread that is running this code.
     // If this thread has been created by the foreign code we will have to
     // create a new one here.
@@ -683,9 +532,6 @@ static void callbackEntryPt(ffi_cif *cif, void *ret, void* args[], void *data)
     pairHandle->WordP()->Set(0, argHandle->Word());
     pairHandle->WordP()->Set(1, resHandle->Word());
 
-    // TODO: This calls BuildCodeSegment to allocate small stub code.
-    // They could easily be cached in X86TaskData::SetCallbackFunction at least
-    // up to the next GC.
     taskData->EnterCallbackFunction(mlEntryHandle, pairHandle);
 
     taskData->saveVec.reset(mark);
@@ -694,6 +540,7 @@ static void callbackEntryPt(ffi_cif *cif, void *ret, void* args[], void *data)
     processes->ThreadReleaseMLMemory(taskData);
 }
 
+
 class PolyFFI: public RtsModule
 {
 public:
@@ -721,3 +568,70 @@ Handle poly_ffi(TaskData *taskData, Handle args, Handle code)
     raise_exception_string(taskData, EXC_foreign, "The foreign function interface is not available on this platform");
 }
 #endif
+
+// General interface to IO.  Ideally the various cases will be made into
+// separate functions.
+POLYUNSIGNED PolyFFIGeneral(PolyObject *threadId, PolyWord code, PolyWord arg)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedCode = taskData->saveVec.push(code);
+    Handle pushedArg = taskData->saveVec.push(arg);
+    Handle result = 0;
+
+    try {
+        result = poly_ffi(taskData, pushedArg, pushedCode);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset);
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+// These functions are needed in the compiler
+POLYUNSIGNED PolySizeFloat()
+{
+    return TAGGED(ffi_type_float.size).AsUnsigned();
+}
+
+POLYUNSIGNED PolySizeDouble()
+{
+    return TAGGED(ffi_type_double.size).AsUnsigned();
+}
+
+// Get either errno or GetLastError
+POLYUNSIGNED PolyFFIGetError(PolyWord addr)
+{
+#if (defined(_WIN32) && ! defined(__CYGWIN__))
+    addr.AsObjPtr()->Set(0, PolyWord::FromUnsigned(GetLastError()));
+#else
+    addr.AsObjPtr()->Set(0, PolyWord::FromUnsigned((POLYUNSIGNED)errno));
+#endif
+    return 0;
+}
+
+// The argument is a SysWord.word value i.e. the address of a byte cell.
+POLYUNSIGNED PolyFFISetError(PolyWord err)
+{
+#if (defined(_WIN32) && ! defined(__CYGWIN__))
+    SetLastError((DWORD)(err.AsObjPtr()->Get(0).AsUnsigned()));
+#else
+    errno = err.AsObjPtr()->Get(0).AsSigned();
+#endif
+    return 0;
+}
+
+struct _entrypts polyFFIEPT[] =
+{
+    { "PolyFFIGeneral",                 (polyRTSFunction)&PolyFFIGeneral},
+    { "PolySizeFloat",                  (polyRTSFunction)&PolySizeFloat},
+    { "PolySizeDouble",                 (polyRTSFunction)&PolySizeDouble},
+    { "PolyFFIGetError",                (polyRTSFunction)&PolyFFIGetError},
+    { "PolyFFISetError",                (polyRTSFunction)&PolyFFISetError},
+
+    { NULL, NULL} // End of list.
+};
+
