@@ -2,7 +2,7 @@
     Title:      Thread functions
     Author:     David C.J. Matthews
 
-    Copyright (c) 2007,2008,2013-15 David C.J. Matthews
+    Copyright (c) 2007,2008,2013-15, 2017 David C.J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -219,6 +219,8 @@ public:
     virtual void ThreadUseMLMemory(TaskData *taskData); 
     virtual void ThreadReleaseMLMemory(TaskData *taskData);
 
+    virtual poly_exn* GetInterrupt(void) { return interrupt_exn; }
+
     // If the schedule lock is already held we need to use these functions.
     void ThreadUseMLMemoryWithSchedLock(TaskData *taskData);
     void ThreadReleaseMLMemoryWithSchedLock(TaskData *taskData);
@@ -314,8 +316,13 @@ public:
     // Shutdown locking.
     void CrowBarFn(void);
     PLock shutdownLock;
-    PCondVar crowbarLock, crowbarStopped;
+    PCondVar crowbarLock;
     bool crowbarRunning;
+#if (defined(HAVE_PTHREAD))
+    pthread_t crowBarThreadId;
+#elif defined(HAVE_WINDOWS_H)
+    HANDLE hCrowBarThread;
+#endif
 
 #ifdef HAVE_WINDOWS_H
     // Used in profiling
@@ -337,6 +344,10 @@ Processes::Processes(): singleThreaded(false), taskArray(0), taskArraySize(0),
     threadRequest(0), exitResult(0), exitRequest(false),
     crowbarRunning(false), sigTask(0)
 {
+#if (defined(HAVE_PTHREAD))
+#elif defined(HAVE_WINDOWS_H)
+    hCrowBarThread = NULL;
+#endif
 #ifdef HAVE_WINDOWS_H
     Waiter::hWakeupEvent = NULL;
     hStopEvent = NULL;
@@ -825,9 +836,7 @@ TaskData::TaskData(): allocPointer(0), allocLimit(0), allocSize(MIN_HEAP_SIZE),
 #ifdef HAVE_WINDOWS_H
     threadHandle = 0;
 #endif
-#ifdef HAVE_PTHREAD
     threadExited = false;
-#endif
 }
 
 TaskData::~TaskData()
@@ -923,11 +932,7 @@ void Processes::ThreadExit(TaskData *taskData)
 
     schedLock.Lock();
     ThreadReleaseMLMemoryWithSchedLock(taskData); // Allow a GC if it was waiting for us.
-    // Remove this from the taskArray
-    unsigned index = get_C_unsigned(taskData, taskData->threadObject->index);
-    ASSERT(index < taskArraySize && taskArray[index] == taskData);
-    taskArray[index] = 0;
-    delete(taskData);
+    taskData->threadExited = true;
     initialThreadWait.Signal(); // Tell it we've finished.
     schedLock.Unlock();
 #ifdef HAVE_PTHREAD
@@ -1447,8 +1452,7 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
         // allocate when we need to raise it.
         // We can only do this once the taskData object has been created.
         if (interrupt_exn == 0)
-            interrupt_exn =
-                DEREFEXNHANDLE(make_exn(taskData, EXC_interrupt, taskData->saveVec.push(TAGGED(0))));
+            interrupt_exn = makeExceptionPacket(taskData, EXC_interrupt);
 
         if (singleThreaded)
         {
@@ -1460,17 +1464,8 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
         schedLock.Lock();
         int errorCode = 0;
 #ifdef HAVE_PTHREAD
-        // Create a thread that isn't joinable since we don't want to wait
-        // for it to finish.
-        pthread_attr_t attrs;
-        pthread_attr_init(&attrs);
-        pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
-        // N.B.  Because we create the thread detached the thread ID will be deleted
-        // by the thread itself so may be invalid at any time, unlike the Windows handle.
-        pthread_t pthreadId;
-        if (pthread_create(&pthreadId, &attrs, NewThreadFunction, taskData) != 0)
+        if (pthread_create(&taskData->threadId, NULL, NewThreadFunction, taskData) != 0)
             errorCode = errno;
-        pthread_attr_destroy(&attrs);
 #elif defined(HAVE_WINDOWS_H)
         taskData->threadHandle =
             CreateThread(NULL, 0, NewThreadFunction, taskData, 0, NULL);
@@ -1513,21 +1508,17 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
                     // It must be running - interrupt it if we are waiting.
                     if (threadRequest != 0) p->InterruptCode();
                 }
-                else
+                else if (p->threadExited) // Has the thread terminated?
                 {
-                    // Has the thread terminated in foreign code?
-                    bool thread_killed = false;
+                    // Wait for it to actually stop then delete the task data.
 #ifdef HAVE_PTHREAD
-                    thread_killed = p->threadExited;
+                    pthread_join(p->threadId, NULL);
 #elif defined(HAVE_WINDOWS_H)
-                    thread_killed = WaitForSingleObject(p->threadHandle, 0) == WAIT_OBJECT_0;
+                    WaitForSingleObject(p->threadHandle, INFINITE);
 #endif
-                    if (thread_killed)
-                    {
-                        delete(p);
-                        taskArray[i] = 0;
-                        globalStats.decCount(PSC_THREADS);
-                    }
+                    delete(p);
+                    taskArray[i] = 0;
+                    globalStats.decCount(PSC_THREADS);
                 }
             }
         }
@@ -1608,9 +1599,15 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
     if (crowbarRunning)
     {
         crowbarLock.Signal();
-        crowbarStopped.Wait(&shutdownLock);
+        shutdownLock.Unlock();
+        // Wait for the thread to terminate.
+#if (defined(HAVE_PTHREAD))
+        pthread_join(crowBarThreadId, NULL);
+#elif defined(HAVE_WINDOWS_H)
+        WaitForSingleObject(hCrowBarThread, 10000);
+#endif
     }
-    shutdownLock.Unlock(); // So it's unlocked when we delete it
+    else shutdownLock.Unlock(); // So it's always unlocked when it's destroyed.
     finish(exitResult); // Close everything down and exit.
 }
 
@@ -1689,14 +1686,7 @@ Handle Processes::ForkThread(TaskData *taskData, Handle threadFunction,
         bool success = false;
         schedLock.Lock();
 #ifdef HAVE_PTHREAD
-        // Create a thread that isn't joinable since we don't want to wait
-        // for it to finish.
-        pthread_attr_t attrs;
-        pthread_attr_init(&attrs);
-        pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
-        pthread_t pthreadId;
-        success = pthread_create(&pthreadId, &attrs, NewThreadFunction, newTaskData) == 0;
-        pthread_attr_destroy(&attrs);
+        success = pthread_create(&newTaskData->threadId, NULL, NewThreadFunction, newTaskData) == 0;
 #elif defined(HAVE_WINDOWS_H)
         newTaskData->threadHandle =
             CreateThread(NULL, 0, NewThreadFunction, newTaskData, 0, NULL);
@@ -1872,23 +1862,21 @@ void Processes::TestAnyEvents(TaskData *taskData)
         throw IOException();
 }
 
-// Stop.  Usually called by one of the threads but
-// in the Windows version can also be called by the GUI or
-// it can be called from the default console interrupt handler.
-// This is more complicated than it seems.  We must avoid
-// calling exit while there are other threads running because
-// exit will finalise the modules and deallocate memory etc.
-// However some threads may be deadlocked or we may be in the
-// middle of a very slow GC and we just want it to stop.
+// Stop.  Exit is usually called by one of the ML threads but
+// in the Windows version can also be called by the GUI.
+// The normal shut-down routine is to wake up the main thread
+// and have it wait until all the ML threads have exited.  This
+// "crow-bar" thread is intended to force a shut-down if that
+// doesn't happen within 40s.  It has been increased from 20s because
+// of a report that this was insufficient on a heavily loaded machine.
 void Processes::CrowBarFn(void)
 {
 #if (defined(HAVE_PTHREAD) || defined(HAVE_WINDOWS_H))
     shutdownLock.Lock();
     crowbarRunning = true;
-    if (crowbarLock.WaitFor(&shutdownLock, 20000)) // Wait for 20s
+    if (crowbarLock.WaitFor(&shutdownLock, 40000)) // Wait for 40s
     {
         // We've been woken by the main thread.  Let it do the shutdown.
-        crowbarStopped.Signal();
         shutdownLock.Unlock();
     }
     else
@@ -1921,21 +1909,21 @@ void Processes::Exit(int n)
     if (singleThreaded)
         finish(n);
 
-    // Start a crowbar thread.  This will stop everything if the main thread
-    // does not reach the point of stopping within 5 seconds.
+#if (defined(HAVE_PTHREAD) || defined(HAVE_WINDOWS_H))
+    {
+        // Start a crowbar thread.  This will stop everything if the main thread
+        // does not reach the point of stopping within 5 seconds.
+        PLocker l(&shutdownLock);
+        if (!crowbarRunning)
+        {
 #if (defined(HAVE_PTHREAD))
-    // Create a thread that isn't joinable since we don't want to wait
-    // for it to finish.
-    pthread_attr_t attrs;
-    pthread_attr_init(&attrs);
-    pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
-    pthread_t threadId;
-    (void)pthread_create(&threadId, &attrs, crowBarFn, 0);
-    pthread_attr_destroy(&attrs);
+            crowbarRunning = pthread_create(&crowBarThreadId, NULL, crowBarFn, 0) == 0;
 #elif defined(HAVE_WINDOWS_H)
-    DWORD dwThrdId;
-    HANDLE hCrowBarThread = CreateThread(NULL, 0, crowBarFn, 0, 0, &dwThrdId);
-    CloseHandle(hCrowBarThread); // Not needed
+            hCrowBarThread = CreateThread(NULL, 0, crowBarFn, 0, 0, NULL);
+            crowbarRunning = hCrowBarThread != NULL;
+#endif
+        }
+    }
 #endif
     // We may be in an interrupt handler with schedLock held.
     // Just set the exit request and go.
@@ -2129,12 +2117,13 @@ void Processes::SignalArrived(void)
 
 #ifdef HAVE_PTHREAD
 // This is called when the thread exits in foreign code and
-// ThreadExit has not been called.  Normally the thread-specific
-// data is cleared.
+// ThreadExit has not been called.
 static void threaddata_destructor(void *p)
 {
     TaskData *pt = (TaskData *)p;
     pt->threadExited = true;
+    // This doesn't actually wake the main thread and relies on the
+    // regular check to release the task data.
 }
 #endif
 
