@@ -435,12 +435,16 @@ void SaveFixupAddress::ScanCodeSpace(CodeSpace *space)
 // Called by the root thread to actually save the state and write the file.
 void SaveRequest::Perform()
 {
+    if (debugOptions & DEBUG_SAVING)
+        Log("SAVE: Beginning saving state.\n");
     // Check that we aren't overwriting our own parent.
     for (unsigned q = 0; q < newHierarchy-1; q++) {
         if (sameFile(hierarchyTable[q]->fileName, fileName))
         {
             errorMessage = "File being saved is used as a parent of this file";
             errCode = 0;
+            if (debugOptions & DEBUG_SAVING)
+                Log("SAVE: File being saved is used as a parent of this file.\n");
             return;
         }
     }
@@ -452,6 +456,8 @@ void SaveRequest::Perform()
     {
         errorMessage = "Cannot open save file";
         errCode = ERRORNUMBER;
+        if (debugOptions & DEBUG_SAVING)
+            Log("SAVE: Cannot open save file.\n");
         return;
     }
 
@@ -464,13 +470,20 @@ void SaveRequest::Perform()
         for (std::vector<PermanentMemSpace*>::iterator i = gMem.pSpaces.begin(); i < gMem.pSpaces.end(); i++)
         {
             PermanentMemSpace *space = *i;
-            if (space->isMutable && ! space->noOverwrite && ! space->byteOnly)
+            if (space->isMutable && !space->noOverwrite && !space->byteOnly)
+            {
+                if (debugOptions & DEBUG_SAVING)
+                    Log("SAVE: Scanning permanent mutable area %p allocated at %p size %lu\n",
+                        space, space->bottom, space->spaceSize());
                 copyScan.ScanAddressesInRegion(space->bottom, space->top);
+            }
         }
     }
     catch (MemoryException &)
     {
         success = false;
+        if (debugOptions & DEBUG_SAVING)
+            Log("SAVE: Scan of permanent mutable area raised memory exception.\n");
     }
 
     // Copy the areas into the export object.  Make sufficient space for
@@ -523,6 +536,9 @@ void SaveRequest::Perform()
 
     exports.memTableEntries = memTableCount;
 
+    if (debugOptions & DEBUG_SAVING)
+        Log("SAVE: Updating references to moved objects.\n");
+
     // Update references to moved objects.
     SaveFixupAddress fixup;
     for (std::vector<LocalMemSpace*>::iterator i = gMem.lSpaces.begin(); i < gMem.lSpaces.end(); i++)
@@ -561,10 +577,14 @@ void SaveRequest::Perform()
     // Try to promote the spaces even if we've had a failure because export
     // spaces are deleted in ~CopyScan and we may have already copied
     // some objects there.
+    if (debugOptions & DEBUG_SAVING)
+        Log("SAVE: Promoting export spaces to permanent spaces.\n");
     if (! gMem.PromoteExportSpaces(newHierarchy) || ! success)
     {
         errorMessage = "Out of Memory";
         errCode = NOMEMORY;
+        if (debugOptions & DEBUG_SAVING)
+            Log("SAVE: Unable to promote export spaces.\n");
         return;
     }
     // Remove any deeper entries from the hierarchy table.
@@ -575,6 +595,9 @@ void SaveRequest::Perform()
         hierarchyTable[hierarchyDepth] = 0;
     }
 
+    if (debugOptions & DEBUG_SAVING)
+        Log("SAVE: Writing out data.\n");
+
     // Write out the file header.
     SavedStateHeader saveHeader;
     memset(&saveHeader, 0, sizeof(saveHeader));
@@ -674,6 +697,9 @@ void SaveRequest::Perform()
     fwrite(&saveHeader, sizeof(saveHeader), 1, exports.exportFile);
     fwrite(descrs, sizeof(SavedStateSegmentDescr), exports.memTableEntries, exports.exportFile);
 
+    if (debugOptions & DEBUG_SAVING)
+        Log("SAVE: Writing complete.\n");
+
     // Add an entry to the hierarchy table for this file.
     (void)AddHierarchyEntry(fileName, saveHeader.timeStamp);
 
@@ -778,28 +804,106 @@ void ClearWeakByteRef::ScanAddressesInObject(PolyObject *base, POLYUNSIGNED leng
     }
 }
 
+// This is copied from the B-tree in MemMgr.  It probably should be
+// merged but will do for the moment.  It's intended to reduce the
+// cost of finding the segment for relocation.
+
+class SpaceBTree
+{
+public:
+    SpaceBTree(bool is, unsigned i = 0) : isLeaf(is), index(i) { }
+    virtual ~SpaceBTree() {}
+
+    bool isLeaf;
+    unsigned index; // The index if this is a leaf
+};
+
+// A non-leaf node in the B-tree
+class SpaceBTreeTree : public SpaceBTree
+{
+public:
+    SpaceBTreeTree();
+    virtual ~SpaceBTreeTree();
+
+    SpaceBTree *tree[256];
+};
+
+SpaceBTreeTree::SpaceBTreeTree() : SpaceBTree(false)
+{
+    for (unsigned i = 0; i < 256; i++)
+        tree[i] = 0;
+}
+
+SpaceBTreeTree::~SpaceBTreeTree()
+{
+    for (unsigned i = 0; i < 256; i++)
+        delete(tree[i]);
+}
+
+
 // This class is used to relocate addresses in areas that have been loaded.
 class LoadRelocate
 {
 public:
-    LoadRelocate(): descrs(0), targetAddresses(0), nDescrs(0), errorMessage(0) {}
+    LoadRelocate(): descrs(0), targetAddresses(0), nDescrs(0), errorMessage(0), spaceTree(0) {}
     ~LoadRelocate();
 
     void RelocateObject(PolyObject *p);
     void RelocateAddressAt(PolyWord *pt);
+    void AddTreeRange(SpaceBTree **t, unsigned index, uintptr_t startS, uintptr_t endS);
 
     SavedStateSegmentDescr *descrs;
     PolyWord **targetAddresses;
     unsigned nDescrs;
     const char *errorMessage;
+    SpaceBTree *spaceTree;
 };
 
 LoadRelocate::~LoadRelocate()
 {
     if (descrs) delete[](descrs);
     if (targetAddresses) delete[](targetAddresses);
+    delete(spaceTree);
 }
 
+// Add an entry to the space B-tree.
+void LoadRelocate::AddTreeRange(SpaceBTree **tt, unsigned index, uintptr_t startS, uintptr_t endS)
+{
+    if (*tt == 0)
+        *tt = new SpaceBTreeTree;
+    ASSERT(!(*tt)->isLeaf);
+    SpaceBTreeTree *t = (SpaceBTreeTree*)*tt;
+
+    const unsigned shift = (sizeof(void*) - 1) * 8; // Takes the high-order byte
+    uintptr_t r = startS >> shift;
+    ASSERT(r < 256);
+    const uintptr_t s = endS == 0 ? 256 : endS >> shift;
+    ASSERT(s >= r && s <= 256);
+
+    if (r == s) // Wholly within this entry
+        AddTreeRange(&(t->tree[r]), index, startS << 8, endS << 8);
+    else
+    {
+        // Deal with any remainder at the start.
+        if ((r << shift) != startS)
+        {
+            AddTreeRange(&(t->tree[r]), index, startS << 8, 0 /*End of range*/);
+            r++;
+        }
+        // Whole entries.
+        while (r < s)
+        {
+            ASSERT(t->tree[r] == 0);
+            t->tree[r] = new SpaceBTree(true, index);
+            r++;
+        }
+        // Remainder at the end.
+        if ((s << shift) != endS)
+            AddTreeRange(&(t->tree[r]), index, 0, endS << 8);
+    }
+}
+
+
 // Update the addresses in a group of words.
 void LoadRelocate::RelocateAddressAt(PolyWord *pt)
 {
@@ -808,26 +912,33 @@ void LoadRelocate::RelocateAddressAt(PolyWord *pt)
     if (val.IsTagged()) return;
 
     // Which segment is this address in?
-    unsigned i;
-    for (i = 0; i < nDescrs; i++)
+    // N.B. As with SpaceForAddress we need to subtract 1 to point to the length word.
+    uintptr_t t = (uintptr_t)(val.AsStackAddr() - 1);
+    SpaceBTree *tr = spaceTree;
+
+    // Each level of the tree is either a leaf or a vector of trees.
+    unsigned j = sizeof(void *) * 8;
+    for (;;)
     {
-        SavedStateSegmentDescr *descr = &descrs[i];
-        if (val.AsAddress() > descr->originalAddress &&
-            val.AsAddress() <= (char*)descr->originalAddress + descr->segmentSize)
-        {
+        if (tr == 0) break;
+        if (tr->isLeaf) {
             // It's in this segment: relocate it to the current position.
+            unsigned i = tr->index;
+            SavedStateSegmentDescr *descr = &descrs[i];
             PolyWord *newAddress = targetAddresses[descr->segmentIndex];
+            ASSERT(val.AsAddress() > descr->originalAddress &&
+                val.AsAddress() <= (char*)descr->originalAddress + descr->segmentSize);
             ASSERT(newAddress != 0);
             byte *setAddress = (byte*)newAddress + ((char*)val.AsAddress() - (char*)descr->originalAddress);
             *pt = PolyWord::FromCodePtr(setAddress);
-            break;
+            return;
         }
+        j -= 8;
+        tr = ((SpaceBTreeTree*)tr)->tree[(t >> j) & 0xff];
     }
-    if (i == nDescrs)
-    {
-        // Error: Not found.
-        errorMessage = "Unmatched address";
-    }
+
+    // Error: Not found.
+    errorMessage = "Unmatched address";
 }
 
 // This is based on Exporter::relocateObject but does the reverse.
@@ -996,8 +1107,12 @@ bool StateLoader::LoadFile(bool isInitial, time_t requiredStamp, PolyWord tail)
     {
         unsigned maxIndex = 0;
         for (unsigned i = 0; i < relocate.nDescrs; i++)
+        {
             if (relocate.descrs[i].segmentIndex > maxIndex)
                 maxIndex = relocate.descrs[i].segmentIndex;
+            relocate.AddTreeRange(&relocate.spaceTree, i, (uintptr_t)relocate.descrs[i].originalAddress,
+                (uintptr_t)((char*)relocate.descrs[i].originalAddress + relocate.descrs[i].segmentSize-1));
+        }
         relocate.targetAddresses = new PolyWord*[maxIndex+1];
         for (unsigned i = 0; i <= maxIndex; i++) relocate.targetAddresses[i] = 0;
     }
