@@ -1,12 +1,11 @@
 /*
     Title:  savestate.h - Save and Load state
 
-    Copyright (c) 2007 David C.J. Matthews
+    Copyright (c) 2007, 2015 David C.J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,7 @@ class TaskData;
 Handle SaveState(TaskData *taskData, Handle args);
 
 // Load a saved state file and any ancestors.
-Handle LoadState(TaskData *taskData, Handle hFileName);
+Handle LoadState(TaskData *taskData, bool isHierarchy, Handle hFileName);
 
 // Show the hierarchy.
 Handle ShowHierarchy(TaskData *taskData);
@@ -41,5 +40,11 @@ Handle RenameParent(TaskData *taskData, Handle args);
 // Return the name of the immediate parent stored in a child
 Handle ShowParent(TaskData *taskData, Handle hFileName);
 
+// Write out a module.
+Handle StoreModule(TaskData *taskData, Handle args);
+
+// Load a module.
+Handle LoadModule(TaskData *taskData, Handle args);
+
 #endif
 
