@@ -2,12 +2,11 @@
     Title:      Signal handling
     Author:     David C.J. Matthews
 
-    Copyright (c) 2000-8 David C.J. Matthews
+    Copyright (c) 2000-8, 2016 David C.J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -78,6 +77,11 @@
 #define USE_PTHREAD_SIGNALS 1
 #endif
 
+#if (defined(_WIN32) && ! defined(__CYGWIN__))
+#define INVALIDSIGNAL ERROR_INVALID_PARAMETER
+#else
+#define INVALIDSIGNAL EINVAL
+#endif
 /*
 Signal handling is complicated in a multi-threaded environment.
 The pthread mutex and condition variables are not safe to use in a
@@ -104,11 +108,17 @@ int sigaltstack(const stack_t *, stack_t *);
 #include "gc.h" // For convertedWeak
 #include "scanaddrs.h"
 #include "locking.h"
+#include "rtsentry.h"
 
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
 #include "Console.h"
 #endif
 
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolySetSignalHandler(PolyObject *threadId, PolyWord signalNo, PolyWord action);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyWaitForSignal(PolyObject *threadId);
+}
+
 #define SAVE(x) taskData->saveVec.push(x)
 #define SIZEOF(x) (sizeof(x)/sizeof(word))
 
@@ -225,41 +235,79 @@ void SignalRequest::Perform()
 }
 #endif
 
-
-/* CALL_IO2(Sig_dispatch_,IND) */
-/* This function behaves fairly similarly to the Unix and Windows signal
-   handler.  It takes a signal number and a handler which may be a function
-   or may be 0 (default) or 1 (ignore) and returns the value corresponding
-   to the previous handler. 
-   I've used a general dispatch function here to allow for future expansion. */
-Handle Sig_dispatch_c(TaskData *taskData, Handle args, Handle code)
+static Handle waitForSignal(TaskData *taskData)
 {
-    int c = get_C_int(taskData, DEREFWORDHANDLE(code));
-    switch (c)
+    while (true)
     {
-    case 0: /* Set up signal handler. */
+        processes->ProcessAsynchRequests(taskData); // Check for kill.
+        sigLock.Lock();
+        // Any pending signals?
+        for (int sig = 0; sig < NSIG; sig++)
+        {
+            if (sigData[sig].signalCount > 0)
+            {
+                sigData[sig].signalCount--;
+                if (!IS_INT(findHandler(sig))) /* If it's not DEFAULT or IGNORE. */
+                {
+                    // Create a pair of the handler and signal and pass
+                    // them back to be run.
+                    Handle pair = alloc_and_save(taskData, 2);
+                    // Have to call findHandler again here because that
+                    // allocation could have garbage collected.
+                    DEREFHANDLE(pair)->Set(0, findHandler(sig));
+                    DEREFHANDLE(pair)->Set(1, TAGGED(sig));
+                    sigLock.Unlock();
+                    return pair;
+                }
+            }
+        }
+        if (convertedWeak)
+        {
+            // Last GC converted a weak SOME into NONE.  This isn't
+            // anything to do with signals but the signal thread can
+            // deal with this.
+            sigLock.Unlock();
+            convertedWeak = false;
+            return SAVE(TAGGED(0));
+        }
+        // No pending signal.  Wait until we're woken up.
+        // This releases sigLock after acquiring schedLock.
+        if (! processes->WaitForSignal(taskData, &sigLock))
+            raise_exception_string(taskData, EXC_Fail, "Only one thread may wait for signals");
+    }
+}
+
+POLYUNSIGNED PolySetSignalHandler(PolyObject *threadId, PolyWord signalNo, PolyWord action)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedAction = taskData->saveVec.push(action);
+    Handle oldaction = 0;
+
+    try {
         {
             int sign;
             int action;
-            Handle oldaction;
             {
                 // Lock while we look at the signal vector but release
                 // it before making a root request.
                 PLocker locker(&sigLock);
                 // We have to pass this to the main thread to 
                 // set up the signal handler.
-                sign = get_C_int(taskData, DEREFHANDLE(args)->Get(0));
+                sign = get_C_int(taskData, signalNo);
                 /* Decode the action if it is Ignore or Default. */
-                if (IS_INT(DEREFHANDLE(args)->Get(1)))
-                    action = (int)UNTAGGED(DEREFHANDLE(args)->Get(1));
+                if (pushedAction->Word().IsTagged())
+                    action = (int)pushedAction->Word().UnTagged();
                 else action = HANDLE_SIG; /* Set the handler. */
                 if (sign <= 0 || sign >= NSIG)
-                    raise_syscall(taskData, "Invalid signal value", EINVAL);
+                    raise_syscall(taskData, "Invalid signal value", INVALIDSIGNAL);
 
                 /* Get the old action before updating the vector. */
                 oldaction = SAVE(findHandler(sign));
                 // Now update it.
-                sigData[sign].handler = DEREFWORDHANDLE(args)->Get(1);
+                sigData[sign].handler = pushedAction->Word();
             }
 
             // Request a change in the masking by the root thread.
@@ -272,60 +320,38 @@ Handle Sig_dispatch_c(TaskData *taskData, Handle args, Handle code)
                 processes->MakeRootRequest(taskData, &request);
 #endif
             }
-            return oldaction;
         }
 
-    case 1: // Called by the signal handler thread.  Blocks until a signal
-            // is available.
-        {
-            while (true)
-            {
-                processes->ProcessAsynchRequests(taskData); // Check for kill.
-                sigLock.Lock();
-                // Any pending signals?
-                for (int sig = 0; sig < NSIG; sig++)
-                {
-                    if (sigData[sig].signalCount > 0)
-                    {
-                        sigData[sig].signalCount--;
-                        if (!IS_INT(findHandler(sig))) /* If it's not DEFAULT or IGNORE. */
-                        {
-                            // Create a pair of the handler and signal and pass
-                            // them back to be run.
-                            Handle pair = alloc_and_save(taskData, 2);
-                            // Have to call findHandler again here because that
-                            // allocation could have garbage collected.
-                            DEREFHANDLE(pair)->Set(0, findHandler(sig));
-                            DEREFHANDLE(pair)->Set(1, TAGGED(sig));
-                            sigLock.Unlock();
-                            return pair;
-                        }
-                    }
-                }
-                if (convertedWeak)
-                {
-                    // Last GC converted a weak SOME into NONE.  This isn't
-                    // anything to do with signals but the signal thread can
-                    // deal with this.
-                    sigLock.Unlock();
-                    convertedWeak = false;
-                    return SAVE(TAGGED(0));
-                }
-                // No pending signal.  Wait until we're woken up.
-                // This releases sigLock after acquiring schedLock.
-                if (! processes->WaitForSignal(taskData, &sigLock))
-                    raise_exception_string(taskData, EXC_Fail, "Only one thread may wait for signals");
-            }
-        }
+    } catch (...) { } // If an ML exception is raised
 
-    default:
-        {
-            char msg[100];
-            sprintf(msg, "Unknown signal function: %d", c);
-            raise_exception_string(taskData, EXC_Fail, msg);
-            return 0;
-        }
+    taskData->saveVec.reset(reset);
+    taskData->PostRTSCall();
+    if (oldaction == 0) return TAGGED(0).AsUnsigned();
+    else return oldaction->Word().AsUnsigned();
+
+}
+
+// Called by the signal handler thread.  Blocks until a signal is available.
+POLYUNSIGNED PolyWaitForSignal(PolyObject *threadId)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle result = 0;
+
+    try {
+        result = waitForSignal(taskData);
     }
+    catch (KillException &) {
+        processes->ThreadExit(taskData); // May test for kill
+    }
+    catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset);
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
 }
 
 // Set up per-thread signal data: basically signal stack.
@@ -448,6 +474,14 @@ void init_asyncmask(sigset_t *mask)
 
 #endif
 
+struct _entrypts sigHandlerEPT[] =
+{
+    { "PolySetSignalHandler",           (polyRTSFunction)&PolySetSignalHandler},
+    { "PolyWaitForSignal",              (polyRTSFunction)&PolyWaitForSignal},
+
+    { NULL, NULL} // End of list.
+};
+
 class SigHandler: public RtsModule
 {
 public:
@@ -478,8 +512,7 @@ static void *SignalDetectionThread(void *)
     sigset_t active_signals;    
     sigfillset(&active_signals);
     pthread_sigmask(SIG_SETMASK, &active_signals, NULL);
-    int readSignals[NSIG];
-    memset(readSignals, 0, sizeof(readSignals));
+    int readSignals[NSIG] = {0};
 
     while (true)
     {
