@@ -111,6 +111,11 @@
 #define _T(x) x
 #endif
 
+#include <limits>
+#ifdef max
+#undef max
+#endif
+
 #include "run_time.h"
 #include "sys.h"
 #include "save_vec.h"
@@ -221,9 +226,8 @@ void Statistics::Init()
 #endif
     {
         // If we just want the statistics locally.
-        statMemory = (unsigned char*)malloc(STATS_SPACE);
+        statMemory = (unsigned char*)calloc(STATS_SPACE, sizeof(unsigned char));
         if (statMemory == 0) return;
-        memset(statMemory, 0, STATS_SPACE);
     }
 #endif
     
@@ -669,13 +673,26 @@ Handle Statistics::getRemoteStatistics(TaskData *taskData, POLYUNSIGNED pid)
     return result;
 #elif HAVE_MMAP
     // Find the shared memory in the user's home directory
-    int remMapFd = -1;
-    char remMapFileName[MAXPATHLEN];
-    remMapFileName[0] = 0;
     char *homeDir = getenv("HOME");
     if (homeDir == NULL)
         raise_exception_string(taskData, EXC_Fail, "No statistics available");
-    sprintf(remMapFileName, "%s/.polyml/" POLY_STATS_NAME "%" POLYUFMT, homeDir, pid);
+
+    int remMapFd = -1;
+    size_t remMapSize = 4096;
+    TempCString remMapFileName((char *)malloc(remMapSize));
+    if (remMapFileName == NULL)
+        raise_exception_string(taskData, EXC_Fail, "No statistics available");
+
+    while ((snprintf(remMapFileName, remMapSize, "%s/.polyml/" POLY_STATS_NAME "%" POLYUFMT, homeDir, pid), strlen(remMapFileName) >= remMapSize - 1)) {
+        if (remMapSize > std::numeric_limits<size_t>::max() / 2)
+            raise_exception_string(taskData, EXC_Fail, "No statistics available");
+        remMapSize *= 2;
+        char *newFileName = (char *)realloc(remMapFileName, remMapSize);
+        if (newFileName == NULL)
+            raise_exception_string(taskData, EXC_Fail, "No statistics available");
+        remMapFileName = newFileName;
+    }
+
     remMapFd = open(remMapFileName, O_RDONLY);
     if (remMapFd == -1)
         raise_exception_string(taskData, EXC_Fail, "No statistics available");
