@@ -5,12 +5,11 @@
     Copyright (c) 2000
         Cambridge University Technical Services Limited
 
-    Further development copyright David C.J. Matthews 2011,12
+    Further development copyright David C.J. Matthews 2011,12,16
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    License version 2.1 as published by the Free Software Foundation.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -89,6 +88,9 @@
 
 #ifdef HAVE_ASSERT_H
 #include <assert.h>
+#define ASSERT(x) assert(x)
+#else
+#define ASSERT(x) 0
 #endif
 
 #ifdef HAVE_STDIO_H
@@ -99,6 +101,15 @@
 #include <windows.h>
 #endif
 
+#include <limits>
+// Windows headers define min/max macros, which messes up trying to use std::numeric_limits<T>::min/max()
+#ifdef min
+#undef min
+#endif
+#ifdef max
+#undef max
+#endif
+
 #include "locking.h"
 #include "globals.h"
 #include "arb.h"
@@ -110,6 +121,11 @@
 #include "rts_module.h"
 #include "processes.h"
 #include "heapsizing.h"
+#include "rtsentry.h"
+
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyTimingGeneral(PolyObject *threadId, PolyWord code, PolyWord arg);
+}
 
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
 /* Windows file times are 64-bit numbers representing times in
@@ -143,8 +159,10 @@
 
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
 static FILETIME startTime;
+#define StrToLL _strtoi64
 #else
 static struct timeval startTime;
+#define StrToLL strtoll
 #endif
 
 #if(!(defined(HAVE_GMTIME_R) && defined(HAVE_LOCALTIME_R)))
@@ -153,7 +171,10 @@ static struct timeval startTime;
 static PLock timeLock("Timing");
 #endif
 
-Handle timing_dispatch_c(TaskData *taskData, Handle args, Handle code)
+#define XSTR(X) STR(X)
+#define STR(X)  #X
+
+static Handle timing_dispatch_c(TaskData *taskData, Handle args, Handle code)
 {
     unsigned c = get_C_unsigned(taskData, DEREFWORDHANDLE(code));
     switch (c)
@@ -316,7 +337,7 @@ Handle timing_dispatch_c(TaskData *taskData, Handle args, Handle code)
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
             FILETIME ut, ct, et, kt;
             if (! GetProcessTimes(GetCurrentProcess(), &ct, &et, &kt, &ut))
-                raise_syscall(taskData, "GetProcessTimes failed", 0-GetLastError());
+                raise_syscall(taskData, "GetProcessTimes failed", GetLastError());
             return Make_arb_from_Filetime(taskData, ut);
 #else
             struct rusage rusage;
@@ -332,7 +353,7 @@ Handle timing_dispatch_c(TaskData *taskData, Handle args, Handle code)
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
             FILETIME ct, et, kt, ut;
             if (! GetProcessTimes(GetCurrentProcess(), &ct, &et, &kt, &ut))
-                raise_syscall(taskData, "GetProcessTimes failed", 0-GetLastError());
+                raise_syscall(taskData, "GetProcessTimes failed", GetLastError());
             return Make_arb_from_Filetime(taskData, kt);
 #else
             struct rusage rusage;
@@ -402,6 +423,28 @@ Handle timing_dispatch_c(TaskData *taskData, Handle args, Handle code)
     }
 }
 
+// General interface to timing.  Ideally the various cases will be made into
+// separate functions.
+POLYUNSIGNED PolyTimingGeneral(PolyObject *threadId, PolyWord code, PolyWord arg)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedCode = taskData->saveVec.push(code);
+    Handle pushedArg = taskData->saveVec.push(arg);
+    Handle result = 0;
+
+    try {
+        result = timing_dispatch_c(taskData, pushedArg, pushedCode);
+    } catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset);
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
 #ifdef HAVE_WINDOWS_H
 void addFiletimes(FILETIME *result, const FILETIME *x)
 {
@@ -495,6 +538,13 @@ void TimeValTime::sub(const TimeValTime &f)
 #endif
 
 
+struct _entrypts timingEPT[] =
+{
+    { "PolyTimingGeneral",              (polyRTSFunction)&PolyTimingGeneral},
+
+    { NULL, NULL} // End of list.
+};
+
 class Timing: public RtsModule
 {
 public:
@@ -513,3 +563,36 @@ void Timing::Init(void)
     gettimeofday(&startTime, NULL);
 #endif
 }
+
+time_t getBuildTime(void)
+{
+    char *source_date_epoch = getenv("SOURCE_DATE_EPOCH");
+    if (source_date_epoch) {
+        errno = 0;
+        char *endptr;
+        long long epoch = StrToLL(source_date_epoch, &endptr, 10);
+        if ((errno == ERANGE && (epoch == LLONG_MIN || epoch == LLONG_MAX)) || (errno != 0 && epoch == 0)) {
+            fprintf(stderr, "Environment variable $SOURCE_DATE_EPOCH: " XSTR(StrToLL) ": %s\n", strerror(errno));
+            goto err;
+        }
+        if (endptr == source_date_epoch) {
+            fprintf(stderr, "Environment variable $SOURCE_DATE_EPOCH: No digits were found: %s\n", endptr);
+            goto err;
+        }
+        if (*endptr != '\0') {
+            fprintf(stderr, "Environment variable $SOURCE_DATE_EPOCH: Trailing garbage: %s\n", endptr);
+            goto err;
+        }
+        if (epoch < (long long)std::numeric_limits<time_t>::min()) {
+            fprintf(stderr, "Environment variable $SOURCE_DATE_EPOCH: value must be greater than or equal to: %lld but was found to be: %lld\n", (long long)std::numeric_limits<time_t>::min(), epoch);
+            goto err;
+        }
+        if (epoch > (long long)std::numeric_limits<time_t>::max()) {
+            fprintf(stderr, "Environment variable $SOURCE_DATE_EPOCH: value must be smaller than or equal to: %lld but was found to be: %lld\n", (long long)std::numeric_limits<time_t>::max(), epoch);
+            goto err;
+        }
+        return (time_t) epoch;
+    }
+err:
+    return time(NULL);
+}
