@@ -50,6 +50,13 @@
 
 #include <lmcons.h>
 
+#ifdef HAVE_ASSERT_H
+#include <assert.h>
+#define ASSERT(x) assert(x)
+#else
+#define ASSERT(x) 0
+#endif
+
 
 #include "globals.h"
 #include "arb.h"
@@ -67,12 +74,18 @@
 #include "save_vec.h"
 #include "rts_module.h"
 #include "machine_dep.h"
+#include "rtsentry.h"
 
 #define STREAMID(x) (DEREFSTREAMHANDLE(x)->streamNo)
 
 #define SAVE(x) taskData->saveVec.push(x)
 #define SIZEOF(x) (sizeof(x)/sizeof(word))
 
+extern "C" {
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyOSSpecificGeneral(PolyObject *threadId, PolyWord code, PolyWord arg);
+    POLYEXTERNALSYMBOL POLYUNSIGNED PolyGetOSType();
+}
+
 typedef enum
 {
     HE_UNUSED,
@@ -97,16 +110,14 @@ typedef enum
 typedef struct {
     PolyObject *token; /* pointer into ML heap */
     HANDENTRYTYPE  entryType; /* Type of handle */
-    union {
-        HKEY    hKey; /* Registry key. */
 
-        struct {
-            /* Process and IO channels. */
-            HANDLE hProcess, hInput, hOutput, hEvent;
-            PolyObject *readToken, *writeToken;
-        } process;
-        HCONV hcDDEConv; /* DDE Conversation. */
-    } entry;
+    // Only some of these are used in particular cases
+    // but using a union here complicates things.
+    HKEY    hKey; /* Registry key. */
+    /* Process and IO channels. */
+    HANDLE hProcess, hInput, hOutput, hEvent;
+    PolyWord readToken, writeToken;
+    HCONV hcDDEConv; /* DDE Conversation. */
 } HANDLETAB, *PHANDLETAB;
 
 
@@ -120,22 +131,22 @@ static void close_handle(PHANDLETAB pTab)
     switch (pTab->entryType)
     {
     case HE_REGISTRY:
-        RegCloseKey(pTab->entry.hKey);
+        RegCloseKey(pTab->hKey);
         break;
 
     case HE_PROCESS:
-        if (pTab->entry.process.hProcess)
-            CloseHandle(pTab->entry.process.hProcess);
-        if (pTab->entry.process.hInput != INVALID_HANDLE_VALUE)
-            CloseHandle(pTab->entry.process.hInput);
-        if (pTab->entry.process.hOutput != INVALID_HANDLE_VALUE)
-            CloseHandle(pTab->entry.process.hOutput);
-        if (pTab->entry.process.hEvent)
-            CloseHandle(pTab->entry.process.hEvent);
+        if (pTab->hProcess)
+            CloseHandle(pTab->hProcess);
+        if (pTab->hInput != INVALID_HANDLE_VALUE)
+            CloseHandle(pTab->hInput);
+        if (pTab->hOutput != INVALID_HANDLE_VALUE)
+            CloseHandle(pTab->hOutput);
+        if (pTab->hEvent)
+            CloseHandle(pTab->hEvent);
         break;
 
     case HE_DDECONVERSATION:
-        CloseDDEConversation(pTab->entry.hcDDEConv);
+        CloseDDEConversation(pTab->hcDDEConv);
         break;
     case HE_UNUSED:
         break; // Avoid warnings
@@ -183,7 +194,7 @@ static Handle make_handle_entry(TaskData *taskData)
                 maxHandleTab += maxHandleTab/2;
                 void *p = realloc(handleTable, maxHandleTab*sizeof(HANDLETAB));
                 // If there's insufficient memory leave the old table.
-                if (p == 0) raise_syscall(taskData, "Insufficient memory", ENOMEM);
+                if (p == 0) raise_syscall(taskData, "Insufficient memory", ERROR_NOT_ENOUGH_MEMORY);
                 handleTable = (PHANDLETAB)p;
                 /* Clear the new space. */
                 memset(handleTable+oldMax, 0, (maxHandleTab-oldMax)*sizeof(HANDLETAB));
@@ -297,7 +308,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
     switch (c)
     {
     case 0: /* Return our OS type.  Not in any structure. */
-        return Make_arbitrary_precision(taskData, 1); /* 1 for Windows. */
+        return Make_fixed_precision(taskData, 1); /* 1 for Windows. */
 
         /* Windows-specific functions. */
     case 1000: /* execute */
@@ -319,49 +330,48 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
         {
             PHANDLETAB hnd = get_handle(DEREFWORD(args), HE_PROCESS);
             if (hnd == 0)
-                raise_syscall(taskData, "Process is closed", EINVAL);
+                raise_syscall(taskData, "Process is closed", ERROR_INVALID_HANDLE);
             // Close the streams. Either of them may have been
             // passed to the stream package.
-            if (hnd->entry.process.hInput != INVALID_HANDLE_VALUE)
-                CloseHandle(hnd->entry.process.hInput);
-            hnd->entry.process.hInput = INVALID_HANDLE_VALUE;
-            if (hnd->entry.process.hEvent)
-                CloseHandle(hnd->entry.process.hEvent);
-            hnd->entry.process.hEvent = NULL;
-            if (hnd->entry.process.readToken)
+            if (hnd->hInput != INVALID_HANDLE_VALUE)
+                CloseHandle(hnd->hInput);
+            hnd->hInput = INVALID_HANDLE_VALUE;
+            if (hnd->hEvent)
+                CloseHandle(hnd->hEvent);
+            hnd->hEvent = NULL;
+            if (hnd->readToken.IsDataPtr())
             {
                 PIOSTRUCT strm =
-                    get_stream(hnd->entry.process.readToken);
+                    get_stream(hnd->readToken);
                 if (strm != NULL) close_stream(strm);
             }
-            hnd->entry.process.readToken = 0;
-            if (hnd->entry.process.hOutput != INVALID_HANDLE_VALUE)
-                CloseHandle(hnd->entry.process.hOutput);
-            hnd->entry.process.hOutput = INVALID_HANDLE_VALUE;
-            if (hnd->entry.process.writeToken)
+            hnd->readToken = ClosedToken;
+            if (hnd->hOutput != INVALID_HANDLE_VALUE)
+                CloseHandle(hnd->hOutput);
+            hnd->hOutput = INVALID_HANDLE_VALUE;
+            if (hnd->writeToken.IsDataPtr())
             {
                 PIOSTRUCT strm =
-                    get_stream(hnd->entry.process.writeToken);
+                    get_stream(hnd->writeToken);
                 if (strm != NULL) close_stream(strm);
             }
-            hnd->entry.process.writeToken = 0;
+            hnd->writeToken = ClosedToken;
 
             // See if it's finished.
             while (true) {
                 DWORD dwResult;
-                if (GetExitCodeProcess(hnd->entry.process.hProcess, &dwResult) == 0)
-                    raise_syscall(taskData, "GetExitCodeProcess failed",
-                            -(int)GetLastError());
+                if (GetExitCodeProcess(hnd->hProcess, &dwResult) == 0)
+                    raise_syscall(taskData, "GetExitCodeProcess failed", GetLastError());
                 if (dwResult != STILL_ACTIVE) {
                     /* Finished - return the result. */
                     /* Note: we haven't closed the handle because we might want to ask
                        for the result again.  We only close it when we've garbage-collected
                        the token.  Doing this runs the risk of running out of handles.
                        Maybe change it and remember the result in ML. */
-                    return Make_arbitrary_precision(taskData, dwResult);
+                    return Make_fixed_precision(taskData, dwResult);
                 }
                 // Block and try again.
-                WaitHandle waiter(hnd->entry.process.hProcess);
+                WaitHandle waiter(hnd->hProcess);
                 processes->ThreadPauseForIO(taskData, &waiter);
             }
         }
@@ -390,8 +400,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return openRegistryKey(taskData, args, hnd->entry.hKey);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return openRegistryKey(taskData, args, hnd->hKey);
         }
 
     case 1009: // Create a subkey within one of the roots.
@@ -409,15 +419,15 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return createRegistryKey(taskData, args, hnd->entry.hKey);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return createRegistryKey(taskData, args, hnd->hKey);
         }
 
     case 1011: // Close a registry handle.
         {
             PHANDLETAB hnd = get_handle(DEREFWORD(args), HE_REGISTRY);
             if (hnd != 0) close_handle(hnd);
-            return Make_arbitrary_precision(taskData, 0);
+            return Make_fixed_precision(taskData, 0);
         }
 
     case 1012: // Get a value
@@ -435,8 +445,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return queryRegistryKey(taskData, args, hnd->entry.hKey);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return queryRegistryKey(taskData, args, hnd->hKey);
         }
 
     case 1014: // Delete a subkey
@@ -454,8 +464,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return deleteRegistryKey(taskData, args, hnd->entry.hKey);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return deleteRegistryKey(taskData, args, hnd->hKey);
         }
 
     case 1016: // Set a value
@@ -473,8 +483,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return setRegistryKey(taskData, args, hnd->entry.hKey);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return setRegistryKey(taskData, args, hnd->hKey);
         }
 
     case 1018: // Enumerate a key in the predefined keys
@@ -490,8 +500,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return enumerateRegistry(taskData, args, hnd->entry.hKey, TRUE);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return enumerateRegistry(taskData, args, hnd->hKey, TRUE);
         }
 
     case 1020: // Enumerate a value in the predefined keys
@@ -507,8 +517,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return enumerateRegistry(taskData, args, hnd->entry.hKey, FALSE);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return enumerateRegistry(taskData, args, hnd->hKey, FALSE);
         }
 
     case 1022: // Delete a value
@@ -526,8 +536,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             PHANDLETAB hnd =
                 get_handle(DEREFHANDLE(args)->Get(0), HE_REGISTRY);
             if (hnd == 0)
-                raise_syscall(taskData, "Handle is closed", -ERROR_INVALID_HANDLE);
-            return deleteRegistryValue(taskData, args, hnd->entry.hKey);
+                raise_syscall(taskData, "Handle is closed", ERROR_INVALID_HANDLE);
+            return deleteRegistryValue(taskData, args, hnd->hKey);
         }
 
 
@@ -537,8 +547,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             /* Get the file time. */
             getFileTimeFromArb(taskData, args, &ftUTC);
             if (! FileTimeToLocalFileTime(&ftUTC, &ftLocal))
-                raise_syscall(taskData, "FileTimeToLocalFileTime failed",
-                        -(int)GetLastError());
+                raise_syscall(taskData, "FileTimeToLocalFileTime failed", GetLastError());
             return Make_arb_from_Filetime(taskData, ftLocal);
         }
 
@@ -548,8 +557,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             /* Get the file time. */
             getFileTimeFromArb(taskData, args, &ftLocal);
             if (! LocalFileTimeToFileTime(&ftLocal, &ftUTC))
-                raise_syscall(taskData, "LocalFileTimeToFileTime failed",
-                        -(int)GetLastError());
+                raise_syscall(taskData, "LocalFileTimeToFileTime failed", GetLastError());
             return Make_arb_from_Filetime(taskData, ftUTC);
         }
 
@@ -561,13 +569,12 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             Handle resultHandle;
             POLYUNSIGNED length = Poly_string_to_C(DEREFWORD(args), rootName, MAX_PATH);
             if (length > MAX_PATH)
-                raise_syscall(taskData, "Root name too long", ENAMETOOLONG);
+                raise_syscall(taskData, "Root name too long", ERROR_BAD_LENGTH);
             
             if (!GetVolumeInformation(rootName, volName, MAX_PATH,
                     &dwVolSerial, &dwMaxComponentLen, &dwFlags,
                     sysName, MAX_PATH))
-                raise_syscall(taskData, "GetVolumeInformation failed",
-                    -(int)GetLastError());
+                raise_syscall(taskData, "GetVolumeInformation failed", GetLastError());
             volHandle = SAVE(C_string_to_Poly(taskData, volName));
             sysHandle = SAVE(C_string_to_Poly(taskData, sysName));
             serialHandle = Make_arbitrary_precision(taskData, dwVolSerial);
@@ -586,11 +593,19 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             POLYUNSIGNED length = Poly_string_to_C(DEREFWORD(args), fileName, MAX_PATH);
             HINSTANCE hInst;
             if (length > MAX_PATH)
-                raise_syscall(taskData, "File name too long", ENAMETOOLONG);
+                raise_syscall(taskData, "File name too long", ERROR_BAD_LENGTH);
             hInst = FindExecutable(fileName, NULL, execName);
             if ((POLYUNSIGNED)hInst <= 32)
             {
-               raise_syscall(taskData, "FindExecutable failed", -(int)(POLYUNSIGNED)hInst);
+                int error = 0;
+                switch ((POLYUNSIGNED)hInst) {
+                case SE_ERR_FNF: error = ERROR_FILE_NOT_FOUND; break;
+                case SE_ERR_PNF: error = ERROR_PATH_NOT_FOUND; break;
+                case SE_ERR_ACCESSDENIED: error = ERROR_ACCESS_DENIED; break;
+                case SE_ERR_OOM: error = ERROR_NOT_ENOUGH_MEMORY; break;
+                case SE_ERR_NOASSOC: error = ERROR_NO_ASSOCIATION; break;
+                }
+                raise_syscall(taskData, "FindExecutable failed", error);
             }
             return SAVE(C_string_to_Poly(taskData, execName));
         }
@@ -607,8 +622,8 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             BOOL fRes = ShellExecuteEx(&shellEx);
             free((void*)shellEx.lpFile);
             if (! fRes)
-                raise_syscall(taskData, "ShellExecuteEx failed", 0-GetLastError());
-            return Make_arbitrary_precision(taskData, 0);
+                raise_syscall(taskData, "ShellExecuteEx failed", GetLastError());
+            return Make_fixed_precision(taskData, 0);
         }
 
     case 1035: // Launch an application.
@@ -624,12 +639,12 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             free((void*)shellEx.lpFile);
             free((void*)shellEx.lpParameters);
             if (! fRes)
-                raise_syscall(taskData, "ShellExecuteEx failed", 0-GetLastError());
-            return Make_arbitrary_precision(taskData, 0);
+                raise_syscall(taskData, "ShellExecuteEx failed", GetLastError());
+            return Make_fixed_precision(taskData, 0);
         }
 
     case 1036: // Does the process have its own console?
-        return Make_arbitrary_precision(taskData, hMainWindow != NULL ? 1: 0);
+        return Make_fixed_precision(taskData, hMainWindow != NULL ? 1: 0);
 
     case 1037: // Simple execute.
         return simpleExecute(taskData, args);
@@ -651,7 +666,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             handToken = make_handle_entry(taskData);
             pTab = &handleTable[STREAMID(handToken)];
             pTab->entryType = HE_DDECONVERSATION;
-            pTab->entry.hcDDEConv = hcDDEConv;
+            pTab->hcDDEConv = hcDDEConv;
             return handToken;
         }
 
@@ -667,7 +682,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             }
             command = Poly_string_to_C_alloc(args->WordP()->Get(1));
             /* Send a request to the main thread to do the work. */
-            res = ExecuteDDE(command, hnd->entry.hcDDEConv);
+            res = ExecuteDDE(command, hnd->hcDDEConv);
             free(command);
             if (res == -1) raise_syscall(taskData, "DdeClientTransaction failed", 0);
             else return Make_arbitrary_precision(taskData, res);
@@ -677,7 +692,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
         {
             PHANDLETAB hnd = get_handle(args->Word(), HE_DDECONVERSATION);
             if (hnd != 0) close_handle(hnd);
-            return Make_arbitrary_precision(taskData, 0);
+            return Make_fixed_precision(taskData, 0);
         }
 
 
@@ -689,11 +704,11 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             osver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
             // GetVersionEx is deprecated in Windows 8.1
             if (! GetVersionEx(&osver))
-                raise_syscall(taskData, "GetVersionEx failed", -(int)GetLastError());
-            Handle major = Make_arbitrary_precision(taskData, osver.dwMajorVersion);
-            Handle minor = Make_arbitrary_precision(taskData, osver.dwMinorVersion);
-            Handle build = Make_arbitrary_precision(taskData, osver.dwBuildNumber);
-            Handle platform = Make_arbitrary_precision(taskData, osver.dwPlatformId);
+                raise_syscall(taskData, "GetVersionEx failed", GetLastError());
+            Handle major = Make_fixed_precision(taskData, osver.dwMajorVersion);
+            Handle minor = Make_fixed_precision(taskData, osver.dwMinorVersion);
+            Handle build = Make_fixed_precision(taskData, osver.dwBuildNumber);
+            Handle platform = Make_fixed_precision(taskData, osver.dwPlatformId);
             Handle version = SAVE(C_string_to_Poly(taskData, osver.szCSDVersion));
             Handle resVal = alloc_and_save(taskData, 5);
             DEREFHANDLE(resVal)->Set(0, DEREFWORDHANDLE(major));
@@ -708,7 +723,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
         {
             TCHAR path[MAX_PATH+1];
             if (GetWindowsDirectory(path, sizeof(path)/sizeof(TCHAR)) == 0)
-                raise_syscall(taskData, "GetWindowsDirectory failed", -(int)GetLastError());
+                raise_syscall(taskData, "GetWindowsDirectory failed", GetLastError());
             return SAVE(C_string_to_Poly(taskData, path));
         }
 
@@ -716,7 +731,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
         {
             TCHAR path[MAX_PATH+1];
             if (GetSystemDirectory(path, sizeof(path)/sizeof(TCHAR)) == 0)
-                raise_syscall(taskData, "GetSystemDirectory failed", -(int)GetLastError());
+                raise_syscall(taskData, "GetSystemDirectory failed", GetLastError());
             return SAVE(C_string_to_Poly(taskData, path));
         }
 
@@ -725,7 +740,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             TCHAR name[MAX_COMPUTERNAME_LENGTH +1];
             DWORD dwSize = MAX_COMPUTERNAME_LENGTH +1;
             if (GetComputerName(name, &dwSize) == 0)
-                raise_syscall(taskData, "GetComputerName failed", -(int)GetLastError());
+                raise_syscall(taskData, "GetComputerName failed", GetLastError());
             return SAVE(C_string_to_Poly(taskData, name));
         }
 
@@ -734,7 +749,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
             TCHAR name[UNLEN +1];
             DWORD dwSize = UNLEN +1;
             if (GetUserName(name, &dwSize) == 0)
-                raise_syscall(taskData, "GetUserName failed", -(int)GetLastError());
+                raise_syscall(taskData, "GetUserName failed", GetLastError());
             return SAVE(C_string_to_Poly(taskData, name));
         }
 
@@ -757,7 +772,7 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
                 // callback to ML.  For this to work a callback must not overwrite "args".
                 BOOL result = PeekMessage(&msg, hwnd, wMsgFilterMin, wMsgFilterMax, PM_NOREMOVE);
                 processes->ThreadUseMLMemory(taskData);
-                if (result) return Make_arbitrary_precision(taskData, 0);
+                if (result) return Make_fixed_precision(taskData, 0);
                 // Pause until a message arrives.
                 processes->ThreadPause(taskData);
             }
@@ -791,6 +806,37 @@ Handle OS_spec_dispatch_c(TaskData *taskData, Handle args, Handle code)
     }
 }
 
+// General interface to Windows OS-specific.  Ideally the various cases will be made into
+// separate functions.
+POLYUNSIGNED PolyOSSpecificGeneral(PolyObject *threadId, PolyWord code, PolyWord arg)
+{
+    TaskData *taskData = TaskData::FindTaskForId(threadId);
+    ASSERT(taskData != 0);
+    taskData->PreRTSCall();
+    Handle reset = taskData->saveVec.mark();
+    Handle pushedCode = taskData->saveVec.push(code);
+    Handle pushedArg = taskData->saveVec.push(arg);
+    Handle result = 0;
+
+    try {
+        result = OS_spec_dispatch_c(taskData, pushedArg, pushedCode);
+    }
+    catch (KillException &) {
+        processes->ThreadExit(taskData); // Call 1005 may test for kill
+    }
+    catch (...) { } // If an ML exception is raised
+
+    taskData->saveVec.reset(reset); // Ensure the save vec is reset
+    taskData->PostRTSCall();
+    if (result == 0) return TAGGED(0).AsUnsigned();
+    else return result->Word().AsUnsigned();
+}
+
+POLYUNSIGNED PolyGetOSType()
+{
+    return TAGGED(1).AsUnsigned(); // Return 1 for Windows
+}
+
 /*
 The Windows version of this is more complicated than the Unix version because
 we can't manipulate the pipe handles in the child process.  Everything has to be
@@ -895,12 +941,12 @@ static Handle execute(TaskData *taskData, Handle args)
     CloseHandle(hReadFromParent);
     CloseHandle(hWriteToParent);
     pTab->entryType = HE_PROCESS;
-    pTab->entry.process.hProcess = processInfo.hProcess;
-    pTab->entry.process.hInput = hReadFromChild;
-    pTab->entry.process.hOutput = hWriteToChild;
-    pTab->entry.process.hEvent = hEvent;
-    pTab->entry.process.readToken = 0;
-    pTab->entry.process.writeToken = 0;
+    pTab->hProcess = processInfo.hProcess;
+    pTab->hInput = hReadFromChild;
+    pTab->hOutput = hWriteToChild;
+    pTab->hEvent = hEvent;
+    pTab->readToken = ClosedToken;
+    pTab->writeToken = ClosedToken;
 
     return(handToken);
 
@@ -915,7 +961,7 @@ error:
         if (hWriteToParent != INVALID_HANDLE_VALUE) CloseHandle(hWriteToParent);
         if (hReadFromChild != INVALID_HANDLE_VALUE) CloseHandle(hReadFromChild);
         if (hEvent) CloseHandle(hEvent);
-        raise_syscall(taskData, lpszError, -err);
+        raise_syscall(taskData, lpszError, err);
         return NULL; // Never reached.
     }
 }
@@ -959,7 +1005,7 @@ static Handle simpleExecute(TaskData *taskData, Handle args)
         free(commandName);
         free(arguments);
         CloseHandle(hNull);
-        raise_syscall(taskData, "CreateProcess failed", -nErr);
+        raise_syscall(taskData, "CreateProcess failed", nErr);
     }
 
     free(commandName);
@@ -973,13 +1019,13 @@ static Handle simpleExecute(TaskData *taskData, Handle args)
     handToken = make_handle_entry(taskData);
     pTab = &handleTable[STREAMID(handToken)];
     pTab->entryType = HE_PROCESS;
-    pTab->entry.process.hProcess = processInfo.hProcess;
+    pTab->hProcess = processInfo.hProcess;
     // We only use the process handle entry.
-    pTab->entry.process.hInput = INVALID_HANDLE_VALUE;
-    pTab->entry.process.hOutput = INVALID_HANDLE_VALUE;
-    pTab->entry.process.hEvent = NULL;
-    pTab->entry.process.readToken = 0;
-    pTab->entry.process.writeToken = 0;
+    pTab->hInput = INVALID_HANDLE_VALUE;
+    pTab->hOutput = INVALID_HANDLE_VALUE;
+    pTab->hEvent = NULL;
+    pTab->readToken = ClosedToken;
+    pTab->writeToken = ClosedToken;
 
     return(handToken);
 }
@@ -991,32 +1037,28 @@ static Handle openProcessHandle(TaskData *taskData, Handle args, BOOL fIsRead, B
     HANDLE hStream;
     int mode = 0, ioBits = 0;
     if (hnd == 0)
-        raise_syscall(taskData, "Process is closed", EINVAL);
+        raise_syscall(taskData, "Process is closed", ERROR_INVALID_HANDLE);
 
-    if (fIsRead) hStream = hnd->entry.process.hInput;
-    else hStream = hnd->entry.process.hOutput;
+    if (fIsRead) hStream = hnd->hInput;
+    else hStream = hnd->hOutput;
     /* I had previously assumed that it wasn't possible to get the
        same stream twice.  The current basis library definition allows
        it but warns it may produce unpredictable results.  For the moment
        we don't allow it because we could get problems with closing
        the same handle twice. */
-#ifdef _WIN32_WCE
-    // Not possible in Windows CE.
-    raise_syscall(taskData, "Process is closed", EBADF);
-    return 0;
-#else
     if (hStream == INVALID_HANDLE_VALUE)
-        raise_syscall(taskData, "Process is closed", EBADF);
+        raise_syscall(taskData, "Process is closed", ERROR_INVALID_HANDLE);
 
     if (fIsRead) { mode = _O_RDONLY; ioBits = IO_BIT_READ; }
     else { mode = 0; ioBits = IO_BIT_WRITE; }
     if (fIsText) mode |= _O_TEXT; else mode |= _O_BINARY;
 
     Handle str_token = make_stream_entry(taskData);
+    if (str_token == NULL) raise_syscall(taskData, "Insufficient memory", ERROR_NOT_ENOUGH_MEMORY);
     PIOSTRUCT strm = &basic_io_vector[STREAMID(str_token)];
     strm->device.ioDesc = _open_osfhandle ((POLYSIGNED) hStream, mode);
     if (strm->device.ioDesc == -1)
-        raise_syscall(taskData, "_open_osfhandle failed", errno);
+        raise_syscall(taskData, "_open_osfhandle failed", _doserrno);
     strm->ioBits = ioBits | IO_BIT_OPEN | IO_BIT_PIPE;
 
     /* The responsibility for closing the handle is passed to
@@ -1024,20 +1066,19 @@ static Handle openProcessHandle(TaskData *taskData, Handle args, BOOL fIsRead, B
        stream entry so that we can close the stream in "reap". */
     if (fIsRead)
     {
-        hnd->entry.process.hInput = INVALID_HANDLE_VALUE;
-        hnd->entry.process.readToken = strm->token;
+        hnd->hInput = INVALID_HANDLE_VALUE;
+        hnd->readToken = strm->token;
         // Pass the "input available" event.
-        strm->hAvailable = hnd->entry.process.hEvent;
-        hnd->entry.process.hEvent = NULL;
+        strm->hAvailable = hnd->hEvent;
+        hnd->hEvent = NULL;
     }
     else
     {
-        hnd->entry.process.hOutput = INVALID_HANDLE_VALUE;
-        hnd->entry.process.writeToken = strm->token;
+        hnd->hOutput = INVALID_HANDLE_VALUE;
+        hnd->writeToken = strm->token;
     }
 
     return str_token;
-#endif
 }
 
 // Open a registry key and make an entry in the table for it.
@@ -1051,18 +1092,18 @@ static Handle openRegistryKey(TaskData *taskData, Handle args, HKEY hkParent)
     REGSAM sam = get_C_unsigned(taskData, DEREFWORDHANDLE(args)->Get(2));
     POLYUNSIGNED length = Poly_string_to_C(args->WordP()->Get(1), keyName, MAX_PATH);
     if (length > MAX_PATH)
-        raise_syscall(taskData, "Key name too long", ENAMETOOLONG);
+        raise_syscall(taskData, "Key name too long", ERROR_BAD_LENGTH);
 
     // Try opening the key.
     lRes = RegOpenKeyEx(hkParent, keyName, 0, sam, &hkey);
     if (lRes != ERROR_SUCCESS)
-        raise_syscall(taskData, "RegOpenKeyEx failed", -lRes);
+        raise_syscall(taskData, "RegOpenKeyEx failed", lRes);
 
     // Make an entry in the table.
     result = make_handle_entry(taskData);
     pTab = &handleTable[STREAMID(result)];
     pTab->entryType = HE_REGISTRY;
-    pTab->entry.hKey = hkey;
+    pTab->hKey = hkey;
     return result;
 }
 
@@ -1079,22 +1120,22 @@ static Handle createRegistryKey(TaskData *taskData, Handle args, HKEY hkParent)
     unsigned opt = get_C_unsigned(taskData, DEREFWORDHANDLE(args)->Get(2));
     POLYUNSIGNED length = Poly_string_to_C(args->WordP()->Get(1), keyName, MAX_PATH);
     if (length > MAX_PATH)
-        raise_syscall(taskData, "Key name too long", ENAMETOOLONG);
+        raise_syscall(taskData, "Key name too long", ERROR_BAD_LENGTH);
 
     // Try opening the key.
     lRes = RegCreateKeyEx(hkParent, keyName, 0, NULL,
                 opt ? REG_OPTION_NON_VOLATILE : REG_OPTION_VOLATILE,
                 sam, NULL, &hkey, &dwDisp);
     if (lRes != ERROR_SUCCESS)
-        raise_syscall(taskData, "RegCreateKeyEx failed", -lRes);
+        raise_syscall(taskData, "RegCreateKeyEx failed", lRes);
 
     // Make an entry in the table.
     keyResult = make_handle_entry(taskData);
     pTab = &handleTable[STREAMID(keyResult)];
     pTab->entryType = HE_REGISTRY;
-    pTab->entry.hKey = hkey;
+    pTab->hKey = hkey;
     // Record whether this was new or old.
-    dispRes = Make_arbitrary_precision(taskData, dwDisp == REG_CREATED_NEW_KEY ? 0: 1);
+    dispRes = Make_fixed_precision(taskData, dwDisp == REG_CREATED_NEW_KEY ? 0: 1);
     /* Return a pair of the disposition and the token. */
     pair = alloc_and_save(taskData, 2);
     DEREFHANDLE(pair)->Set(0, DEREFWORDHANDLE(dispRes));
@@ -1110,14 +1151,14 @@ static Handle deleteRegistryKey(TaskData *taskData, Handle args, HKEY hkParent)
     LONG lRes;
     POLYUNSIGNED length = Poly_string_to_C(args->WordP()->Get(1), keyName, MAX_PATH);
     if (length > MAX_PATH)
-        raise_syscall(taskData, "Key name too long", ENAMETOOLONG);
+        raise_syscall(taskData, "Key name too long", ERROR_BAD_LENGTH);
 
     // Try deleting the key.  
     lRes = RegDeleteKey(hkParent, keyName);
     if (lRes != ERROR_SUCCESS)
         /* Return the error. */
-        raise_syscall(taskData, "RegDeleteKey failed", -lRes);
-    return Make_arbitrary_precision(taskData, 0);
+        raise_syscall(taskData, "RegDeleteKey failed", lRes);
+    return Make_fixed_precision(taskData, 0);
 }
 
 static Handle deleteRegistryValue(TaskData *taskData, Handle args, HKEY hkParent)
@@ -1127,14 +1168,14 @@ static Handle deleteRegistryValue(TaskData *taskData, Handle args, HKEY hkParent
     POLYUNSIGNED length = Poly_string_to_C(args->WordP()->Get(1).AsObjPtr(),
                     keyName, MAX_PATH);
     if (length > MAX_PATH)
-        raise_syscall(taskData, "Key name too long", ENAMETOOLONG);
+        raise_syscall(taskData, "Key name too long", ERROR_BAD_LENGTH);
 
     // Try deleting the value.
     lRes = RegDeleteValue(hkParent, keyName);
     if (lRes != ERROR_SUCCESS)
         /* Return the original error. */
-        raise_syscall(taskData, "RegDeleteValue failed", -lRes);
-    return Make_arbitrary_precision(taskData, 0);
+        raise_syscall(taskData, "RegDeleteValue failed", lRes);
+    return Make_fixed_precision(taskData, 0);
 }
 
 static Handle queryRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
@@ -1147,7 +1188,7 @@ static Handle queryRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
     DWORD dwType;
     POLYUNSIGNED length = Poly_string_to_C(args->WordP()->Get(1), valName, MAX_PATH);
     if (length > MAX_PATH)
-        raise_syscall(taskData, "Value name too long", ENAMETOOLONG);
+        raise_syscall(taskData, "Value name too long", ERROR_BAD_LENGTH);
 
     // How long is the entry?
     lRes = RegQueryValueEx(hkey, valName, 0, NULL, NULL, &valSize);
@@ -1155,7 +1196,7 @@ static Handle queryRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
     // answer here.
     if (lRes == ERROR_MORE_DATA) valSize = 1024; // Guess
     else if (lRes != ERROR_SUCCESS)
-        raise_syscall(taskData, "RegQueryValueEx failed", -lRes);
+        raise_syscall(taskData, "RegQueryValueEx failed", lRes);
     // Allocate that much store and get the value.  We could
     // try reading directly into ML store to save copying but
     // it hardly seems worthwhile.
@@ -1168,7 +1209,7 @@ static Handle queryRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
             if (newAlloc == 0)
             {
                 free(keyValue);
-                raise_syscall(taskData, "Insufficient memory", ENOMEM);
+                raise_syscall(taskData, "Insufficient memory", ERROR_NOT_ENOUGH_MEMORY);
             }
             keyValue = newAlloc;
             lRes = RegQueryValueEx(hkey, valName, 0, &dwType, keyValue, &valSize);
@@ -1180,7 +1221,7 @@ static Handle queryRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
         if (lRes != ERROR_SUCCESS)
         {
             free(keyValue);
-            raise_syscall(taskData, "RegQueryValue failed", -lRes);
+            raise_syscall(taskData, "RegQueryValue failed", lRes);
         }
         // If we have a string we have to convert this to ANSI/utf-8.
         if (dwType == REG_SZ || dwType == REG_MULTI_SZ || dwType == REG_EXPAND_SZ)
@@ -1190,7 +1231,7 @@ static Handle queryRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
     }
 
     /* Create a pair containing the type and the value. */
-    resType = Make_arbitrary_precision(taskData, dwType);
+    resType = Make_fixed_precision(taskData, dwType);
     result = alloc_and_save(taskData, 2);
     DEREFHANDLE(result)->Set(0, DEREFWORDHANDLE(resType));
     DEREFHANDLE(result)->Set(1, DEREFWORDHANDLE(resVal));
@@ -1205,7 +1246,7 @@ static Handle setRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
     POLYUNSIGNED length = Poly_string_to_C(args->WordP()->Get(1), valName, MAX_PATH);
     DWORD dwType = get_C_unsigned(taskData, DEREFWORDHANDLE(args)->Get(2));
     if (length > MAX_PATH)
-        raise_syscall(taskData, "Value name too long", ENAMETOOLONG);
+        raise_syscall(taskData, "Value name too long", ERROR_BAD_LENGTH);
 
     // The value is binary.  Strings will already have had a null added.
     if (IS_INT(str))
@@ -1222,9 +1263,9 @@ static Handle setRegistryKey(TaskData *taskData, Handle args, HKEY hkey)
     }
 
     if (lRes != ERROR_SUCCESS)
-        raise_syscall(taskData, "RegSetValue failed", -lRes);
+        raise_syscall(taskData, "RegSetValue failed", lRes);
 
-    return Make_arbitrary_precision(taskData, 0);
+    return Make_fixed_precision(taskData, 0);
 }
 
 // Enumerate a key or a value.  Returns a string option containing NONE if
@@ -1241,13 +1282,13 @@ static Handle enumerateRegistry(TaskData *taskData, Handle args, HKEY hkey, BOOL
         FILETIME ftMod;
         lRes = RegEnumKeyEx(hkey, num, keyName, &dwLength, NULL, NULL, NULL, &ftMod);
         if (lRes != ERROR_SUCCESS && lRes != ERROR_NO_MORE_ITEMS)
-            raise_syscall(taskData, "RegEnumKeyEx failed", -lRes);
+            raise_syscall(taskData, "RegEnumKeyEx failed", lRes);
     }
     else
     {
         lRes = RegEnumValue(hkey, num, keyName, &dwLength, NULL, NULL, NULL, NULL);
         if (lRes != ERROR_SUCCESS && lRes != ERROR_NO_MORE_ITEMS)
-            raise_syscall(taskData, "RegEnumValue failed", -lRes);
+            raise_syscall(taskData, "RegEnumValue failed", lRes);
     }
     if (lRes == ERROR_NO_MORE_ITEMS)
         return SAVE(NONE_VALUE); /* NONE. */
@@ -1257,6 +1298,14 @@ static Handle enumerateRegistry(TaskData *taskData, Handle args, HKEY hkey, BOOL
     return result;
 }
 
+struct _entrypts osSpecificEPT[] =
+{
+    { "PolyGetOSType",                  (polyRTSFunction)&PolyGetOSType},
+    { "PolyOSSpecificGeneral",          (polyRTSFunction)&PolyOSSpecificGeneral},
+
+    { NULL, NULL} // End of list.
+};
+
 class WindowsModule: public RtsModule
 {
 public:
@@ -1284,10 +1333,18 @@ void WindowsModule::GarbageCollect(ScanAddress *process)
                 /* Update the references to opened streams but
                    do this only as weak references.  If the stream
                    has gone away then that's fine. */
-                if (str->entry.process.readToken)
-                    process->ScanRuntimeAddress(&str->entry.process.readToken, ScanAddress::STRENGTH_WEAK);
-                if (str->entry.process.writeToken)
-                    process->ScanRuntimeAddress(&str->entry.process.writeToken, ScanAddress::STRENGTH_WEAK);
+                if (str->readToken.IsDataPtr())
+                {
+                    PolyObject *token = str->readToken.AsObjPtr();
+                    process->ScanRuntimeAddress(&token, ScanAddress::STRENGTH_WEAK);
+                    str->readToken = token == 0 ? ClosedToken : token;
+                }
+                if (str->writeToken.IsDataPtr())
+                {
+                    PolyObject *token = str->writeToken.AsObjPtr();
+                    process->ScanRuntimeAddress(&token, ScanAddress::STRENGTH_WEAK);
+                    str->writeToken = token == 0 ? ClosedToken : token;
+                }
             }
             process->ScanRuntimeAddress(&str->token, ScanAddress::STRENGTH_WEAK);
             /* Unreferenced entries may return zero. */ 
